/**
 * Admin-facing JavaScript for CargoHub
 *
 * Handles interactions on the admin settings pages
 */
(function($) {
    'use strict';

    /**
     * Admin functionality
     */
    var CargoHubAdmin = {
        
        /**
         * Initialize the admin scripts
         */
        init: function() {
            this.initTabs();
            this.initFormHandlers();
            this.initDismissNotices();
            this.initTooltips();
            this.initSettingsConditionals();
        },
        
        /**
         * Initialize tabs on settings pages
         */
        initTabs: function() {
            $('.cargohub-admin-tabs').each(function() {
                var $tabs = $(this);
                var $tabLinks = $tabs.find('.cargohub-tab-link');
                var $tabContents = $('.cargohub-tab-content');
                
                $tabLinks.on('click', function(e) {
                    e.preventDefault();
                    
                    var tabId = $(this).attr('href');
                    
                    // Update active tab
                    $tabLinks.removeClass('nav-tab-active');
                    $(this).addClass('nav-tab-active');
                    
                    // Show the selected tab content
                    $tabContents.hide();
                    $(tabId).show();
                    
                    // Update URL hash
                    if (history.pushState) {
                        history.pushState(null, null, tabId);
                    } else {
                        location.hash = tabId;
                    }
                });
                
                // Check if a tab is specified in the URL
                var hash = window.location.hash;
                if (hash) {
                    $tabs.find('a[href="' + hash + '"]').click();
                } else {
                    // Activate the first tab by default
                    $tabLinks.first().click();
                }
            });
        },
        
        /**
         * Initialize form handlers
         */
        initFormHandlers: function() {
            // Handle form submissions via AJAX
            $('.cargohub-ajax-form').on('submit', function(e) {
                e.preventDefault();
                
                var $form = $(this);
                var $submitButton = $form.find('button[type="submit"]');
                var originalButtonText = $submitButton.text();
                
                // Disable submit button and show loading
                $submitButton.prop('disabled', true).text(cargohub_admin.i18n.saving);
                
                $.ajax({
                    type: 'POST',
                    url: cargohub_admin.ajax_url,
                    data: $form.serialize() + '&action=cargohub_admin_action',
                    success: function(response) {
                        if (response.success) {
                            CargoHubAdmin.showMessage(response.data.message, 'success');
                        } else {
                            CargoHubAdmin.showMessage(response.data.message || cargohub_admin.i18n.error, 'error');
                        }
                    },
                    error: function() {
                        CargoHubAdmin.showMessage(cargohub_admin.i18n.error, 'error');
                    },
                    complete: function() {
                        // Re-enable submit button
                        $submitButton.prop('disabled', false).text(originalButtonText);
                    }
                });
            });
            
            // Handle delete confirmation
            $('.cargohub-admin').on('click', '.cargohub-delete-item', function(e) {
                if (!confirm(cargohub_admin.i18n.confirm_delete)) {
                    e.preventDefault();
                }
            });
        },
        
        /**
         * Initialize dismiss notices
         */
        initDismissNotices: function() {
            // Make WordPress notices dismissible
            $('.cargohub-admin .notice').each(function() {
                if (!$(this).hasClass('is-dismissible')) {
                    $(this).addClass('is-dismissible');
                }
            });
            
            // Handle dismiss button click
            $(document).on('click', '.notice.is-dismissible .notice-dismiss', function() {
                $(this).closest('.notice').fadeOut(300, function() {
                    $(this).remove();
                });
            });
        },
        
        /**
         * Initialize tooltips
         */
        initTooltips: function() {
            $('.cargohub-tooltip').each(function() {
                var $tooltip = $(this);
                var $tooltipText = $tooltip.find('.cargohub-tooltip-text');
                
                $tooltip.on('mouseenter', function() {
                    $tooltipText.show();
                });
                
                $tooltip.on('mouseleave', function() {
                    $tooltipText.hide();
                });
            });
        },
        
        /**
         * Initialize conditional settings
         */
        initSettingsConditionals: function() {
            // Volumetric weight toggle
            $('#enable_volumetric_weight').on('change', function() {
                var $volumetricDivisor = $('#volumetric_divisor').closest('tr');
                
                if ($(this).is(':checked')) {
                    $volumetricDivisor.show();
                } else {
                    $volumetricDivisor.hide();
                }
            }).trigger('change');
            
            // COD option toggle
            $('#enable_cod').on('change', function() {
                var $codFeePercentage = $('#cod_fee_percentage').closest('tr');
                
                if ($(this).is(':checked')) {
                    $codFeePercentage.show();
                } else {
                    $codFeePercentage.hide();
                }
            }).trigger('change');
        },
        
        /**
         * Show a message to the user
         */
        showMessage: function(message, type) {
            var $notice = $('<div class="notice is-dismissible"><p></p></div>');
            
            if (type === 'success') {
                $notice.addClass('notice-success');
            } else if (type === 'error') {
                $notice.addClass('notice-error');
            } else if (type === 'warning') {
                $notice.addClass('notice-warning');
            } else if (type === 'info') {
                $notice.addClass('notice-info');
            }
            
            $notice.find('p').text(message);
            
            // Add dismiss button
            $notice.append('<button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button>');
            
            // Insert notice at the top of the page
            $('#wpbody-content').prepend($notice);
            
            // Auto-dismiss after 5 seconds for success messages
            if (type === 'success') {
                setTimeout(function() {
                    $notice.fadeOut(300, function() {
                        $(this).remove();
                    });
                }, 5000);
            }
        },
        
        /**
         * Format a price according to store settings
         */
        formatPrice: function(price) {
            price = parseFloat(price).toFixed(cargohub_admin.decimals);
            
            // Replace decimal separator
            price = price.replace('.', cargohub_admin.decimal_separator);
            
            // Add thousand separator
            if (cargohub_admin.thousand_separator) {
                price = price.replace(/\B(?=(\d{3})+(?!\d))/g, cargohub_admin.thousand_separator);
            }
            
            // Add currency symbol
            if (cargohub_admin.currency_position === 'left') {
                return cargohub_admin.currency_symbol + price;
            } else if (cargohub_admin.currency_position === 'right') {
                return price + cargohub_admin.currency_symbol;
            } else if (cargohub_admin.currency_position === 'left_space') {
                return cargohub_admin.currency_symbol + ' ' + price;
            } else if (cargohub_admin.currency_position === 'right_space') {
                return price + ' ' + cargohub_admin.currency_symbol;
            }
            
            return cargohub_admin.currency_symbol + price;
        }
    };
    
    // Initialize on document ready
    $(document).ready(function() {
        CargoHubAdmin.init();
    });
    
})(jQuery);