/**
 * Public-facing JavaScript for CargoHub
 *
 * Handles real-time price calculations and form interactions
 */
(function($) {
    'use strict';

    // Store our variables
    var CargoHub = {
        form: null,
        weightField: null,
        lengthField: null,
        widthField: null,
        heightField: null,
        serviceField: null,
        codField: null,
        priceDisplay: null,
        priceTable: null,
        priceContainer: null,
        quantityField: null,
        
        // Settings from localized script
        settings: {
            ajaxUrl: '',
            nonce: '',
            volumetricDivisor: 5000,
            enableVolumetric: true,
            weightUnit: 'kg',
            dimensionUnit: 'cm',
            codFeePercentage: 10,
            currencySymbol: '$',
            currencyPosition: 'left',
            decimalSeparator: '.',
            thousandSeparator: ',',
            decimals: 2
        },
        
        /**
         * Initialize the script
         */
        init: function() {
            // Cache DOM elements
            this.form = $('.cargohub-dimension-fields').closest('form');
            this.weightField = $('#cargohub_weight');
            this.lengthField = $('#cargohub_length');
            this.widthField = $('#cargohub_width');
            this.heightField = $('#cargohub_height');
            this.serviceField = $('#cargohub_service_type');
            this.codField = $('#cargohub_cod_option');
            
            // If any of the main fields are missing, don't continue
            if (!this.form.length || !this.weightField.length || !this.serviceField.length) {
                return;
            }
            
            // Find the quantity field if it exists
            this.quantityField = this.form.find('input.qty');
            
            // Create price display elements if they don't exist
            this.createPriceDisplay();
            
            // Bind events
            this.bindEvents();
            
            // Initial calculation
            this.calculatePrice();
        },
        
        /**
         * Create the price display container
         */
        createPriceDisplay: function() {
            // If it already exists, return
            if ($('.cargohub-price-calculation').length) {
                this.priceContainer = $('.cargohub-price-calculation');
                this.priceTable = this.priceContainer.find('.cargohub-price-table');
                return;
            }
            
            // Create the container
            this.priceContainer = $('<div>', {
                'class': 'cargohub-price-calculation',
                'html': '<h4>' + cargoHubPublic.i18n.priceCalculation + '</h4>'
            });
            
            // Create the table
            this.priceTable = $('<table>', {
                'class': 'cargohub-price-table',
                'html': '<tbody></tbody>'
            });
            
            // Add the table to the container
            this.priceContainer.append(this.priceTable);
            
            // Add the container after the last cargohub field
            $('.cargohub-cod-option').after(this.priceContainer);
            
            // Initially hide it until we have a calculation
            this.priceContainer.hide();
        },
        
        /**
         * Bind events to form elements
         */
        bindEvents: function() {
            var self = this;
            
            // Input change events
            this.weightField.on('input change', function() {
                self.calculatePrice();
            });
            
            this.lengthField.on('input change', function() {
                self.calculatePrice();
            });
            
            this.widthField.on('input change', function() {
                self.calculatePrice();
            });
            
            this.heightField.on('input change', function() {
                self.calculatePrice();
            });
            
            this.serviceField.on('change', function() {
                self.calculatePrice();
            });
            
            this.codField.on('change', function() {
                self.calculatePrice();
            });
            
            // If quantity field exists, bind to it as well
            if (this.quantityField.length) {
                this.quantityField.on('input change', function() {
                    // For courier products, always force quantity to 1
                    $(this).val(1);
                });
            }
        },
        
        /**
         * Calculate price based on input values
         */
        calculatePrice: function() {
            var self = this;
            
            // Get input values
            var weight = parseFloat(this.weightField.val()) || 0;
            var length = parseFloat(this.lengthField.val()) || 0;
            var width = parseFloat(this.widthField.val()) || 0;
            var height = parseFloat(this.heightField.val()) || 0;
            var serviceType = this.serviceField.val() || '';
            var codOption = this.codField.is(':checked');
            
            // Basic validation
            if (weight <= 0 || length <= 0 || width <= 0 || height <= 0 || serviceType === '') {
                this.priceContainer.hide();
                return;
            }
            
            // Calculate volumetric weight if enabled
            var volumetricWeight = 0;
            var chargeableWeight = weight;
            
            if (this.settings.enableVolumetric) {
                volumetricWeight = (length * width * height) / this.settings.volumetricDivisor;
                chargeableWeight = Math.max(weight, volumetricWeight);
            }
            
            // Prepare the AJAX data
            var data = {
                action: 'cargohub_calculate_price',
                weight: weight,
                length: length,
                width: width,
                height: height,
                volumetric_weight: volumetricWeight,
                chargeable_weight: chargeableWeight,
                service_type: serviceType,
                cod_option: codOption ? 'yes' : 'no',
                security: this.settings.nonce
            };
            
            // Show loading state
            this.priceContainer.addClass('loading').show();
            this.priceTable.find('tbody').html(
                '<tr><td colspan="2">' + cargoHubPublic.i18n.calculating + '</td></tr>'
            );
            
            // Make the AJAX call
            $.ajax({
                url: this.settings.ajaxUrl,
                type: 'POST',
                data: data,
                success: function(response) {
                    if (response.success) {
                        self.updatePriceDisplay(response.data);
                    } else {
                        self.showError(response.data.message || cargoHubPublic.i18n.errorCalculating);
                    }
                },
                error: function() {
                    self.showError(cargoHubPublic.i18n.errorCalculating);
                },
                complete: function() {
                    self.priceContainer.removeClass('loading');
                }
            });
        },
        
        /**
         * Update the price display with calculated values
         */
        updatePriceDisplay: function(data) {
            var html = '';
            
            // Base shipping cost
            html += '<tr>' +
                    '<th>' + cargoHubPublic.i18n.shippingCost + '</th>' +
                    '<td>' + this.formatPrice(data.shipping_cost) + '</td>' +
                    '</tr>';
            
            // Show volumetric weight if used
            if (this.settings.enableVolumetric && data.volumetric_weight > data.weight) {
                html += '<tr>' +
                        '<th>' + cargoHubPublic.i18n.actualWeight + '</th>' +
                        '<td>' + data.weight + ' ' + this.settings.weightUnit + '</td>' +
                        '</tr>';
                        
                html += '<tr>' +
                        '<th>' + cargoHubPublic.i18n.volumetricWeight + '</th>' +
                        '<td>' + data.volumetric_weight + ' ' + this.settings.weightUnit + '</td>' +
                        '</tr>';
            }
            
            // COD fee if applicable
            if (data.cod_option === 'yes' && data.cod_fee > 0) {
                html += '<tr>' +
                        '<th>' + cargoHubPublic.i18n.codFee + ' (' + this.settings.codFeePercentage + '%)</th>' +
                        '<td>' + this.formatPrice(data.cod_fee) + '</td>' +
                        '</tr>';
            }
            
            // Total cost
            html += '<tr class="cargohub-price-total">' +
                    '<th>' + cargoHubPublic.i18n.totalCost + '</th>' +
                    '<td>' + this.formatPrice(data.total_cost) + '</td>' +
                    '</tr>';
            
            // Update the table content
            this.priceTable.find('tbody').html(html);
            this.priceContainer.show();
        },
        
        /**
         * Show an error message in the price display
         */
        showError: function(message) {
            this.priceTable.find('tbody').html(
                '<tr><td colspan="2" class="cargohub-error">' + message + '</td></tr>'
            );
            this.priceContainer.show();
        },
        
        /**
         * Format a price according to store settings
         */
        formatPrice: function(price) {
            var formattedPrice = parseFloat(price).toFixed(this.settings.decimals);
            
            // Replace decimal separator
            formattedPrice = formattedPrice.replace('.', this.settings.decimalSeparator);
            
            // Add thousand separator
            formattedPrice = this.addThousandSeparator(formattedPrice);
            
            // Add currency symbol
            if (this.settings.currencyPosition === 'left') {
                return this.settings.currencySymbol + formattedPrice;
            } else if (this.settings.currencyPosition === 'right') {
                return formattedPrice + this.settings.currencySymbol;
            } else if (this.settings.currencyPosition === 'left_space') {
                return this.settings.currencySymbol + ' ' + formattedPrice;
            } else if (this.settings.currencyPosition === 'right_space') {
                return formattedPrice + ' ' + this.settings.currencySymbol;
            }
            
            return this.settings.currencySymbol + formattedPrice;
        },
        
        /**
         * Add thousand separator to a number
         */
        addThousandSeparator: function(price) {
            var parts = price.split(this.settings.decimalSeparator);
            parts[0] = parts[0].replace(/\B(?=(\d{3})+(?!\d))/g, this.settings.thousandSeparator);
            return parts.join(this.settings.decimalSeparator);
        }
    };
    
    // Initialize on document ready
    $(document).ready(function() {
        // Apply settings from the localized script
        if (typeof cargoHubPublic !== 'undefined') {
            $.extend(CargoHub.settings, cargoHubPublic);
        }
        
        // Initialize the script
        CargoHub.init();
    });

})(jQuery);