<?php
/**
 * The admin-specific order functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The admin-specific order functionality of the plugin.
 *
 * Handles order management on the admin side.
 */
class CargoHub_Admin_Orders {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param    string    $plugin_name       The name of this plugin.
     * @param    string    $version           The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        
        // Add custom order columns
        add_filter('manage_edit-shop_order_columns', array($this, 'add_order_columns'), 20);
        add_action('manage_shop_order_posts_custom_column', array($this, 'render_order_columns'), 20, 2);
        
        // Add order filters
        add_action('restrict_manage_posts', array($this, 'add_order_filters'));
        add_filter('request', array($this, 'filter_orders'));
        
        // Add meta boxes to order admin page
        add_action('add_meta_boxes', array($this, 'add_order_meta_boxes'));
        
        // Save courier details when order is saved
        add_action('woocommerce_process_shop_order_meta', array($this, 'save_courier_details'), 10, 2);
    }

    /**
     * Add custom columns to the orders list.
     *
     * @since    1.0.0
     * @param    array    $columns    The current order columns.
     * @return   array               The modified order columns.
     */
    public function add_order_columns($columns) {
        $new_columns = array();
        
        // Insert at specific position (after order status)
        foreach ($columns as $key => $column) {
            $new_columns[$key] = $column;
            
            if ($key === 'order_status') {
                $new_columns['courier_service'] = __('Courier Service', 'cargohub');
                $new_columns['package_weight'] = __('Weight', 'cargohub');
            }
        }
        
        return $new_columns;
    }

    /**
     * Render the custom column content.
     *
     * @since    1.0.0
     * @param    string    $column       The column name.
     * @param    int       $order_id     The order ID.
     */
    public function render_order_columns($column, $order_id) {
        switch ($column) {
            case 'courier_service':
                $order = wc_get_order($order_id);
                $service_type = $order->get_meta('_cargohub_service_type', true);
                
                if (!empty($service_type)) {
                    // Get service type name from database
                    global $wpdb;
                    $table_name = $wpdb->prefix . 'cargohub_service_types';
                    $service_name = $wpdb->get_var($wpdb->prepare(
                        "SELECT service_name FROM $table_name WHERE service_code = %s",
                        $service_type
                    ));
                    
                    if ($service_name) {
                        echo esc_html($service_name);
                    } else {
                        echo esc_html($service_type);
                    }
                    
                    // Show COD badge if applicable
                    $cod_option = $order->get_meta('_cargohub_cod_option', true);
                    if ($cod_option === 'yes') {
                        echo ' <span class="cargohub-cod-badge">' . esc_html__('COD', 'cargohub') . '</span>';
                    }
                } else {
                    echo '—';
                }
                break;
                
            case 'package_weight':
                $order = wc_get_order($order_id);
                $weight = $order->get_meta('_cargohub_weight', true);
                $volumetric_weight = $order->get_meta('_cargohub_volumetric_weight', true);
                $weight_unit = $this->get_weight_unit();
                
                if (!empty($weight)) {
                    echo esc_html(wc_format_decimal($weight, 2)) . ' ' . esc_html($weight_unit);
                    
                    // Show volumetric weight if it was used for pricing
                    if (!empty($volumetric_weight) && $volumetric_weight > $weight) {
                        echo ' <span class="cargohub-vol-badge" title="' . esc_attr__('Volumetric weight was used for pricing', 'cargohub') . '">';
                        echo esc_html__('VOL', 'cargohub') . ': ' . esc_html(wc_format_decimal($volumetric_weight, 2)) . ' ' . esc_html($weight_unit);
                        echo '</span>';
                    }
                } else {
                    echo '—';
                }
                break;
        }
    }

    /**
     * Add order filters to the orders list.
     *
     * @since    1.0.0
     */
    public function add_order_filters() {
        global $typenow, $wpdb;
        
        if ($typenow != 'shop_order') {
            return;
        }
        
        // Service type filter
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $service_types = $wpdb->get_results("SELECT service_code, service_name FROM $table_name WHERE active = 1 ORDER BY service_name ASC");
        
        if (!empty($service_types)) {
            $current_service = isset($_GET['courier_service']) ? wc_clean(wp_unslash($_GET['courier_service'])) : '';
            ?>
            <select name="courier_service" id="filter-by-courier-service">
                <option value=""><?php esc_html_e('All courier services', 'cargohub'); ?></option>
                <?php foreach ($service_types as $service) : ?>
                    <option value="<?php echo esc_attr($service->service_code); ?>" <?php selected($current_service, $service->service_code); ?>>
                        <?php echo esc_html($service->service_name); ?>
                    </option>
                <?php endforeach; ?>
            </select>
            <?php
        }
        
        // COD filter
        $current_cod = isset($_GET['cod_option']) ? wc_clean(wp_unslash($_GET['cod_option'])) : '';
        ?>
        <select name="cod_option" id="filter-by-cod-option">
            <option value=""><?php esc_html_e('All payment methods', 'cargohub'); ?></option>
            <option value="yes" <?php selected($current_cod, 'yes'); ?>><?php esc_html_e('COD Only', 'cargohub'); ?></option>
            <option value="no" <?php selected($current_cod, 'no'); ?>><?php esc_html_e('Non-COD Only', 'cargohub'); ?></option>
        </select>
        <?php
    }

    /**
     * Filter orders based on courier filter values.
     *
     * @since    1.0.0
     * @param    array    $vars    The query variables.
     * @return   array            The modified query variables.
     */
    public function filter_orders($vars) {
        global $typenow;
        
        if ($typenow !== 'shop_order') {
            return $vars;
        }
        
        // Filter by service type
        if (!empty($_GET['courier_service'])) {
            $vars['meta_query'][] = array(
                'key'     => '_cargohub_service_type',
                'value'   => wc_clean(wp_unslash($_GET['courier_service'])),
                'compare' => '=',
            );
        }
        
        // Filter by COD option
        if (isset($_GET['cod_option']) && $_GET['cod_option'] !== '') {
            $vars['meta_query'][] = array(
                'key'     => '_cargohub_cod_option',
                'value'   => wc_clean(wp_unslash($_GET['cod_option'])),
                'compare' => '=',
            );
        }
        
        return $vars;
    }

    /**
     * Add meta boxes to the order edit screen.
     *
     * @since    1.0.0
     */
    public function add_order_meta_boxes() {
        add_meta_box(
            'cargohub_order_details',
            __('Courier Details', 'cargohub'),
            array($this, 'render_courier_details_meta_box'),
            'shop_order',
            'side',
            'default'
        );
    }

    /**
     * Render the courier details meta box.
     *
     * @since    1.0.0
     * @param    WP_Post    $post    The post object.
     */
    public function render_courier_details_meta_box($post) {
        $order = wc_get_order($post->ID);
        
        if (!$order) {
            return;
        }
        
        // Get courier details
        $service_type = $order->get_meta('_cargohub_service_type', true);
        $cod_option = $order->get_meta('_cargohub_cod_option', true);
        $weight = $order->get_meta('_cargohub_weight', true);
        $length = $order->get_meta('_cargohub_length', true);
        $width = $order->get_meta('_cargohub_width', true);
        $height = $order->get_meta('_cargohub_height', true);
        $volumetric_weight = $order->get_meta('_cargohub_volumetric_weight', true);
        $weight_unit = $this->get_weight_unit();
        $dimension_unit = $this->get_dimension_unit();
        
        // Check if this is a courier order
        if (empty($service_type) && empty($weight)) {
            echo '<p>' . esc_html__('This is not a courier order.', 'cargohub') . '</p>';
            return;
        }
        
        // Get all service types for dropdown
        global $wpdb;
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $service_types = $wpdb->get_results("SELECT service_code, service_name FROM $table_name WHERE active = 1 ORDER BY service_name ASC");
        
        // Output fields
        wp_nonce_field('cargohub_save_courier_details', 'cargohub_courier_details_nonce');
        ?>
        <div class="cargohub-courier-details">
            <p>
                <label for="cargohub_service_type"><?php esc_html_e('Service Type:', 'cargohub'); ?></label><br />
                <select name="cargohub_service_type" id="cargohub_service_type" class="widefat">
                    <option value=""><?php esc_html_e('-- Select Service Type --', 'cargohub'); ?></option>
                    <?php foreach ($service_types as $type) : ?>
                        <option value="<?php echo esc_attr($type->service_code); ?>" <?php selected($service_type, $type->service_code); ?>>
                            <?php echo esc_html($type->service_name); ?>
                        </option>
                    <?php endforeach; ?>
                </select>
            </p>
            
            <p>
                <label for="cargohub_cod_option"><?php esc_html_e('COD Option:', 'cargohub'); ?></label><br />
                <select name="cargohub_cod_option" id="cargohub_cod_option" class="widefat">
                    <option value="no" <?php selected($cod_option, 'no'); ?>><?php esc_html_e('No', 'cargohub'); ?></option>
                    <option value="yes" <?php selected($cod_option, 'yes'); ?>><?php esc_html_e('Yes', 'cargohub'); ?></option>
                </select>
            </p>
            
            <p>
                <label for="cargohub_weight"><?php esc_html_e('Weight:', 'cargohub'); ?></label><br />
                <input type="number" step="0.01" min="0" name="cargohub_weight" id="cargohub_weight" value="<?php echo esc_attr($weight); ?>" class="widefat" />
                <span class="description"><?php echo esc_html($weight_unit); ?></span>
            </p>
            
            <p>
                <label><?php esc_html_e('Dimensions:', 'cargohub'); ?></label><br />
                <input type="number" step="0.01" min="0" name="cargohub_length" placeholder="<?php esc_attr_e('Length', 'cargohub'); ?>" value="<?php echo esc_attr($length); ?>" style="width: 30%;" />
                <input type="number" step="0.01" min="0" name="cargohub_width" placeholder="<?php esc_attr_e('Width', 'cargohub'); ?>" value="<?php echo esc_attr($width); ?>" style="width: 30%;" />
                <input type="number" step="0.01" min="0" name="cargohub_height" placeholder="<?php esc_attr_e('Height', 'cargohub'); ?>" value="<?php echo esc_attr($height); ?>" style="width: 30%;" />
                <span class="description"><?php echo esc_html($dimension_unit); ?></span>
            </p>
            
            <?php if (!empty($volumetric_weight)) : ?>
            <p>
                <label><?php esc_html_e('Volumetric Weight:', 'cargohub'); ?></label><br />
                <span><?php echo esc_html(wc_format_decimal($volumetric_weight, 2)) . ' ' . esc_html($weight_unit); ?></span>
                <input type="hidden" name="cargohub_volumetric_weight" value="<?php echo esc_attr($volumetric_weight); ?>" />
            </p>
            <?php endif; ?>
            
            <div class="clear"></div>
        </div>
        <?php
    }

    /**
     * Save courier details when order is saved.
     *
     * @since    1.0.0
     * @param    int       $order_id    The order ID.
     * @param    WP_Post   $post        The post object.
     */
    public function save_courier_details($order_id, $post) {
        // Check if our nonce is set
        if (!isset($_POST['cargohub_courier_details_nonce'])) {
            return;
        }
        
        // Verify that the nonce is valid
        if (!wp_verify_nonce($_POST['cargohub_courier_details_nonce'], 'cargohub_save_courier_details')) {
            return;
        }
        
        // Get the order object
        $order = wc_get_order($order_id);
        
        if (!$order) {
            return;
        }
        
        // Save service type
        if (isset($_POST['cargohub_service_type'])) {
            $service_type = sanitize_text_field($_POST['cargohub_service_type']);
            $order->update_meta_data('_cargohub_service_type', $service_type);
        }
        
        // Save COD option
        if (isset($_POST['cargohub_cod_option'])) {
            $cod_option = sanitize_text_field($_POST['cargohub_cod_option']);
            $order->update_meta_data('_cargohub_cod_option', $cod_option);
        }
        
        // Save weight
        if (isset($_POST['cargohub_weight'])) {
            $weight = wc_format_decimal($_POST['cargohub_weight']);
            $order->update_meta_data('_cargohub_weight', $weight);
        }
        
        // Save dimensions
        if (isset($_POST['cargohub_length'])) {
            $length = wc_format_decimal($_POST['cargohub_length']);
            $order->update_meta_data('_cargohub_length', $length);
        }
        
        if (isset($_POST['cargohub_width'])) {
            $width = wc_format_decimal($_POST['cargohub_width']);
            $order->update_meta_data('_cargohub_width', $width);
        }
        
        if (isset($_POST['cargohub_height'])) {
            $height = wc_format_decimal($_POST['cargohub_height']);
            $order->update_meta_data('_cargohub_height', $height);
        }
        
        // Save volumetric weight if provided
        if (isset($_POST['cargohub_volumetric_weight'])) {
            $volumetric_weight = wc_format_decimal($_POST['cargohub_volumetric_weight']);
            $order->update_meta_data('_cargohub_volumetric_weight', $volumetric_weight);
        } else {
            // Calculate volumetric weight if dimensions are provided
            $length = $order->get_meta('_cargohub_length', true);
            $width = $order->get_meta('_cargohub_width', true);
            $height = $order->get_meta('_cargohub_height', true);
            
            if (!empty($length) && !empty($width) && !empty($height)) {
                $volumetric_divisor = $this->get_volumetric_divisor();
                $volumetric_weight = ($length * $width * $height) / $volumetric_divisor;
                $order->update_meta_data('_cargohub_volumetric_weight', $volumetric_weight);
            }
        }
        
        // Save the order meta
        $order->save();
    }

    /**
     * Get the weight unit from settings.
     *
     * @since    1.0.0
     * @return   string    The weight unit.
     */
    private function get_weight_unit() {
        $options = get_option('cargohub_general_settings', array());
        return isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
    }

    /**
     * Get the dimension unit from settings.
     *
     * @since    1.0.0
     * @return   string    The dimension unit.
     */
    private function get_dimension_unit() {
        $options = get_option('cargohub_general_settings', array());
        return isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
    }

    /**
     * Get the volumetric divisor from settings.
     *
     * @since    1.0.0
     * @return   int    The volumetric divisor.
     */
    private function get_volumetric_divisor() {
        $options = get_option('cargohub_general_settings', array());
        return isset($options['volumetric_divisor']) ? $options['volumetric_divisor'] : 5000;
    }
}