<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two hooks for
 * enqueuing the admin-specific stylesheet and JavaScript.
 */
class CargoHub_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param    string    $plugin_name       The name of this plugin.
     * @param    string    $version           The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {
        wp_enqueue_style(
            $this->plugin_name,
            CARGOHUB_PLUGIN_URL . 'assets/css/cargohub-admin.css',
            array(),
            $this->version,
            'all'
        );
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {
        wp_enqueue_script(
            $this->plugin_name,
            CARGOHUB_PLUGIN_URL . 'assets/js/cargohub-admin.js',
            array('jquery', 'jquery-ui-sortable', 'jquery-ui-datepicker'),
            $this->version,
            false
        );
        
        // Add localized script variables
        wp_localize_script(
            $this->plugin_name,
            'cargohub_admin',
            array(
                'ajax_url' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('cargohub-admin-nonce'),
                'currency_symbol' => get_woocommerce_currency_symbol(),
                'decimal_separator' => wc_get_price_decimal_separator(),
                'thousand_separator' => wc_get_price_thousand_separator(),
                'decimals' => wc_get_price_decimals(),
                'i18n' => array(
                    'confirm_delete' => __('Are you sure you want to delete this item? This action cannot be undone.', 'cargohub'),
                    'no_rates' => __('No rates found. Please add some rates.', 'cargohub'),
                    'save_success' => __('Settings saved successfully.', 'cargohub'),
                    'error' => __('An error occurred. Please try again.', 'cargohub')
                )
            )
        );
    }

    /**
     * Add menu items to the admin dashboard.
     *
     * @since    1.0.0
     */
    public function add_admin_menu() {
        // Add top level menu
        add_menu_page(
            __('CargoHub', 'cargohub'),
            __('CargoHub', 'cargohub'),
            'manage_woocommerce',
            'cargohub',
            array($this, 'display_dashboard_page'),
            'dashicons-cart',
            57 // After WooCommerce menu
        );
        
        // Dashboard
        add_submenu_page(
            'cargohub',
            __('Dashboard', 'cargohub'),
            __('Dashboard', 'cargohub'),
            'manage_woocommerce',
            'cargohub',
            array($this, 'display_dashboard_page')
        );
        
        // Orders (redirects to WooCommerce orders)
        add_submenu_page(
            'cargohub',
            __('Orders', 'cargohub'),
            __('Orders', 'cargohub'),
            'manage_woocommerce',
            'cargohub-orders',
            array($this, 'redirect_to_woocommerce_orders')
        );
        
        // Service types
        add_submenu_page(
            'cargohub',
            __('Service Types', 'cargohub'),
            __('Service Types', 'cargohub'),
            'manage_woocommerce',
            'cargohub-service-types',
            array($this, 'display_service_types_page')
        );
        
        // Zones
        add_submenu_page(
            'cargohub',
            __('Shipping Zones', 'cargohub'),
            __('Shipping Zones', 'cargohub'),
            'manage_woocommerce',
            'cargohub-zones',
            array($this, 'display_zones_page')
        );
        
        // Rates
        add_submenu_page(
            'cargohub',
            __('Shipping Rates', 'cargohub'),
            __('Shipping Rates', 'cargohub'),
            'manage_woocommerce',
            'cargohub-rates',
            array($this, 'display_rates_page')
        );
        
        // Settings (uses the WP Settings API)
        add_submenu_page(
            'cargohub',
            __('Settings', 'cargohub'),
            __('Settings', 'cargohub'),
            'manage_woocommerce',
            'cargohub-settings',
            array($this, 'display_settings_page')
        );
    }
    
    /**
     * Display the dashboard page.
     *
     * @since    1.0.0
     */
    public function display_dashboard_page() {
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/views/html-admin-dashboard.php';
    }
    
    /**
     * Redirect to WooCommerce orders.
     *
     * @since    1.0.0
     */
    public function redirect_to_woocommerce_orders() {
        wp_redirect(admin_url('edit.php?post_type=shop_order'));
        exit;
    }
    
    /**
     * Display the service types page.
     *
     * @since    1.0.0
     */
    public function display_service_types_page() {
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/views/html-admin-service-types.php';
    }
    
    /**
     * Display the zones page.
     *
     * @since    1.0.0
     */
    public function display_zones_page() {
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/views/html-admin-zones.php';
    }
    
    /**
     * Display the rates page.
     *
     * @since    1.0.0
     */
    public function display_rates_page() {
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/views/html-admin-rates.php';
    }
    
    /**
     * Display the settings page.
     *
     * @since    1.0.0
     */
    public function display_settings_page() {
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/views/html-admin-settings.php';
    }
    
    /**
     * Handle AJAX requests.
     *
     * @since    1.0.0
     */
    public function handle_ajax() {
        // Check nonce for security
        check_ajax_referer('cargohub-admin-nonce', 'nonce');
        
        // Check user capabilities
        if (!current_user_can('manage_woocommerce')) {
            wp_send_json_error(array('message' => __('You do not have permission to perform this action.', 'cargohub')));
        }
        
        $action = isset($_POST['cargohub_action']) ? sanitize_text_field($_POST['cargohub_action']) : '';
        
        switch ($action) {
            case 'save_service_type':
                $this->save_service_type();
                break;
                
            case 'delete_service_type':
                $this->delete_service_type();
                break;
                
            case 'save_zone':
                $this->save_zone();
                break;
                
            case 'delete_zone':
                $this->delete_zone();
                break;
                
            case 'save_rate':
                $this->save_rate();
                break;
                
            case 'delete_rate':
                $this->delete_rate();
                break;
                
            default:
                wp_send_json_error(array('message' => __('Invalid action.', 'cargohub')));
                break;
        }
        
        // If we reach here, something went wrong
        wp_send_json_error(array('message' => __('An error occurred. Please try again.', 'cargohub')));
    }
    
    /**
     * Save a service type.
     *
     * @since    1.0.0
     */
    private function save_service_type() {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $id = isset($_POST['id']) ? intval($_POST['id']) : 0;
        
        $service_data = array(
            'service_name' => isset($_POST['service_name']) ? sanitize_text_field($_POST['service_name']) : '',
            'service_code' => isset($_POST['service_code']) ? sanitize_text_field($_POST['service_code']) : '',
            'description' => isset($_POST['description']) ? sanitize_textarea_field($_POST['description']) : '',
            'active' => isset($_POST['active']) ? 1 : 0
        );
        
        // Validate required fields
        if (empty($service_data['service_name']) || empty($service_data['service_code'])) {
            wp_send_json_error(array('message' => __('Service name and code are required.', 'cargohub')));
        }
        
        if ($id > 0) {
            // Update existing service type
            $result = $wpdb->update(
                $table_name,
                $service_data,
                array('id' => $id)
            );
        } else {
            // Insert new service type
            $result = $wpdb->insert(
                $table_name,
                $service_data
            );
            $id = $wpdb->insert_id;
        }
        
        if ($result === false) {
            wp_send_json_error(array('message' => __('Could not save service type.', 'cargohub')));
        }
        
        wp_send_json_success(array(
            'message' => __('Service type saved successfully.', 'cargohub'),
            'id' => $id
        ));
    }
    
    /**
     * Delete a service type.
     *
     * @since    1.0.0
     */
    private function delete_service_type() {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $id = isset($_POST['id']) ? intval($_POST['id']) : 0;
        
        if ($id <= 0) {
            wp_send_json_error(array('message' => __('Invalid service type ID.', 'cargohub')));
        }
        
        // Check if the service type is in use in any rates
        $rates_table = $wpdb->prefix . 'cargohub_rates';
        $service_code = $wpdb->get_var($wpdb->prepare("SELECT service_code FROM $table_name WHERE id = %d", $id));
        
        $in_use = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $rates_table WHERE service_type = %s", $service_code));
        
        if ($in_use > 0) {
            wp_send_json_error(array('message' => __('This service type cannot be deleted because it is in use in shipping rates.', 'cargohub')));
        }
        
        $result = $wpdb->delete(
            $table_name,
            array('id' => $id)
        );
        
        if ($result === false) {
            wp_send_json_error(array('message' => __('Could not delete service type.', 'cargohub')));
        }
        
        wp_send_json_success(array(
            'message' => __('Service type deleted successfully.', 'cargohub')
        ));
    }
    
    /**
     * Save a zone.
     *
     * @since    1.0.0
     */
    private function save_zone() {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_zones';
        $id = isset($_POST['id']) ? intval($_POST['id']) : 0;
        
        $countries = isset($_POST['countries']) ? (array) $_POST['countries'] : array();
        $countries = array_map('sanitize_text_field', $countries);
        
        $zone_data = array(
            'zone_name' => isset($_POST['zone_name']) ? sanitize_text_field($_POST['zone_name']) : '',
            'countries' => serialize($countries)
        );
        
        // Validate required fields
        if (empty($zone_data['zone_name']) || empty($countries)) {
            wp_send_json_error(array('message' => __('Zone name and at least one country are required.', 'cargohub')));
        }
        
        if ($id > 0) {
            // Update existing zone
            $result = $wpdb->update(
                $table_name,
                $zone_data,
                array('id' => $id)
            );
        } else {
            // Insert new zone
            $result = $wpdb->insert(
                $table_name,
                $zone_data
            );
            $id = $wpdb->insert_id;
        }
        
        if ($result === false) {
            wp_send_json_error(array('message' => __('Could not save zone.', 'cargohub')));
        }
        
        wp_send_json_success(array(
            'message' => __('Zone saved successfully.', 'cargohub'),
            'id' => $id
        ));
    }
    
    /**
     * Delete a zone.
     *
     * @since    1.0.0
     */
    private function delete_zone() {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_zones';
        $id = isset($_POST['id']) ? intval($_POST['id']) : 0;
        
        if ($id <= 0) {
            wp_send_json_error(array('message' => __('Invalid zone ID.', 'cargohub')));
        }
        
        // Check if the zone is in use in any rates
        $rates_table = $wpdb->prefix . 'cargohub_rates';
        $in_use = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM $rates_table WHERE zone_id = %d", $id));
        
        if ($in_use > 0) {
            wp_send_json_error(array('message' => __('This zone cannot be deleted because it is in use in shipping rates.', 'cargohub')));
        }
        
        $result = $wpdb->delete(
            $table_name,
            array('id' => $id)
        );
        
        if ($result === false) {
            wp_send_json_error(array('message' => __('Could not delete zone.', 'cargohub')));
        }
        
        wp_send_json_success(array(
            'message' => __('Zone deleted successfully.', 'cargohub')
        ));
    }
    
    /**
     * Save a shipping rate.
     *
     * @since    1.0.0
     */
    private function save_rate() {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_rates';
        $id = isset($_POST['id']) ? intval($_POST['id']) : 0;
        
        $rate_data = array(
            'weight_from' => isset($_POST['weight_from']) ? wc_format_decimal($_POST['weight_from']) : 0,
            'weight_to' => isset($_POST['weight_to']) ? wc_format_decimal($_POST['weight_to']) : 0,
            'service_type' => isset($_POST['service_type']) ? sanitize_text_field($_POST['service_type']) : '',
            'price' => isset($_POST['price']) ? wc_format_decimal($_POST['price']) : 0,
            'zone_id' => isset($_POST['zone_id']) ? intval($_POST['zone_id']) : 0
        );
        
        // Validate required fields
        if ($rate_data['weight_from'] < 0 || $rate_data['weight_to'] <= 0 || $rate_data['weight_from'] >= $rate_data['weight_to']) {
            wp_send_json_error(array('message' => __('Invalid weight range.', 'cargohub')));
        }
        
        if (empty($rate_data['service_type']) || $rate_data['price'] <= 0 || $rate_data['zone_id'] <= 0) {
            wp_send_json_error(array('message' => __('Service type, price and zone are required.', 'cargohub')));
        }
        
        // Check for overlapping weight ranges for the same service type and zone
        $overlapping = $wpdb->get_var($wpdb->prepare(
            "SELECT COUNT(*) FROM $table_name 
            WHERE service_type = %s 
            AND zone_id = %d 
            AND id != %d
            AND (
                (weight_from <= %f AND weight_to > %f) OR
                (weight_from < %f AND weight_to >= %f) OR
                (weight_from >= %f AND weight_to <= %f)
            )",
            $rate_data['service_type'],
            $rate_data['zone_id'],
            $id,
            $rate_data['weight_from'],
            $rate_data['weight_from'],
            $rate_data['weight_to'],
            $rate_data['weight_to'],
            $rate_data['weight_from'],
            $rate_data['weight_to']
        ));
        
        if ($overlapping > 0) {
            wp_send_json_error(array('message' => __('This weight range overlaps with an existing rate for the same service type and zone.', 'cargohub')));
        }
        
        if ($id > 0) {
            // Update existing rate
            $result = $wpdb->update(
                $table_name,
                $rate_data,
                array('id' => $id)
            );
        } else {
            // Insert new rate
            $result = $wpdb->insert(
                $table_name,
                $rate_data
            );
            $id = $wpdb->insert_id;
        }
        
        if ($result === false) {
            wp_send_json_error(array('message' => __('Could not save shipping rate.', 'cargohub')));
        }
        
        wp_send_json_success(array(
            'message' => __('Shipping rate saved successfully.', 'cargohub'),
            'id' => $id
        ));
    }
    
    /**
     * Delete a shipping rate.
     *
     * @since    1.0.0
     */
    private function delete_rate() {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_rates';
        $id = isset($_POST['id']) ? intval($_POST['id']) : 0;
        
        if ($id <= 0) {
            wp_send_json_error(array('message' => __('Invalid rate ID.', 'cargohub')));
        }
        
        $result = $wpdb->delete(
            $table_name,
            array('id' => $id)
        );
        
        if ($result === false) {
            wp_send_json_error(array('message' => __('Could not delete shipping rate.', 'cargohub')));
        }
        
        wp_send_json_success(array(
            'message' => __('Shipping rate deleted successfully.', 'cargohub')
        ));
    }
}