<?php
/**
 * The settings functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The settings functionality of the plugin.
 *
 * Handles the plugin settings using the WordPress Settings API.
 */
class CargoHub_Settings {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param    string    $plugin_name       The name of this plugin.
     * @param    string    $version           The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the settings using the WordPress Settings API.
     *
     * @since    1.0.0
     */
    public function register_settings() {
        // Register setting
        register_setting(
            'cargohub_settings',
            'cargohub_general_settings',
            array($this, 'validate_general_settings')
        );
        
        // General settings section
        add_settings_section(
            'cargohub_general_section',
            __('General Settings', 'cargohub'),
            array($this, 'render_general_section'),
            'cargohub_settings'
        );
        
        // Weight settings
        add_settings_field(
            'weight_unit',
            __('Weight Unit', 'cargohub'),
            array($this, 'render_weight_unit_field'),
            'cargohub_settings',
            'cargohub_general_section'
        );
        
        add_settings_field(
            'dimension_unit',
            __('Dimension Unit', 'cargohub'),
            array($this, 'render_dimension_unit_field'),
            'cargohub_settings',
            'cargohub_general_section'
        );
        
        add_settings_field(
            'min_weight',
            __('Minimum Weight', 'cargohub'),
            array($this, 'render_min_weight_field'),
            'cargohub_settings',
            'cargohub_general_section'
        );
        
        add_settings_field(
            'max_weight',
            __('Maximum Weight', 'cargohub'),
            array($this, 'render_max_weight_field'),
            'cargohub_settings',
            'cargohub_general_section'
        );
        
        // Volumetric weight settings
        add_settings_field(
            'enable_volumetric_weight',
            __('Enable Volumetric Weight', 'cargohub'),
            array($this, 'render_enable_volumetric_weight_field'),
            'cargohub_settings',
            'cargohub_general_section'
        );
        
        add_settings_field(
            'volumetric_divisor',
            __('Volumetric Divisor', 'cargohub'),
            array($this, 'render_volumetric_divisor_field'),
            'cargohub_settings',
            'cargohub_general_section'
        );
        
        // COD settings
        add_settings_field(
            'cod_fee_percentage',
            __('COD Fee Percentage', 'cargohub'),
            array($this, 'render_cod_fee_percentage_field'),
            'cargohub_settings',
            'cargohub_general_section'
        );
        
        // Product settings section
        add_settings_section(
            'cargohub_product_section',
            __('Product Settings', 'cargohub'),
            array($this, 'render_product_section'),
            'cargohub_settings'
        );
        
        add_settings_field(
            'courier_products',
            __('Courier Products', 'cargohub'),
            array($this, 'render_courier_products_field'),
            'cargohub_settings',
            'cargohub_product_section'
        );
        
        // Email settings section
        add_settings_section(
            'cargohub_email_section',
            __('Email Settings', 'cargohub'),
            array($this, 'render_email_section'),
            'cargohub_settings'
        );
        
        add_settings_field(
            'include_in_emails',
            __('Include in Emails', 'cargohub'),
            array($this, 'render_include_in_emails_field'),
            'cargohub_settings',
            'cargohub_email_section'
        );
        
        // Register display settings (for additional settings not in the general section)
        register_setting(
            'cargohub_display_settings',
            'cargohub_display_settings',
            array($this, 'validate_display_settings')
        );
        
        // Display settings section
        add_settings_section(
            'cargohub_display_section',
            __('Display Settings', 'cargohub'),
            array($this, 'render_display_section'),
            'cargohub_display_settings'
        );
        
        add_settings_field(
            'show_weight_details',
            __('Show Weight Details', 'cargohub'),
            array($this, 'render_show_weight_details_field'),
            'cargohub_display_settings',
            'cargohub_display_section'
        );
    }
    
    /**
     * Validate general settings.
     *
     * @since    1.0.0
     * @param    array    $input    The input array to validate.
     * @return   array              The validated input array.
     */
    public function validate_general_settings($input) {
        $output = array();
        
        // Weight unit
        $output['weight_unit'] = isset($input['weight_unit']) ? sanitize_text_field($input['weight_unit']) : 'kg';
        
        // Dimension unit
        $output['dimension_unit'] = isset($input['dimension_unit']) ? sanitize_text_field($input['dimension_unit']) : 'cm';
        
        // Min weight
        $output['min_weight'] = isset($input['min_weight']) ? wc_format_decimal($input['min_weight']) : 0.5;
        if ($output['min_weight'] < 0) {
            $output['min_weight'] = 0;
        }
        
        // Max weight
        $output['max_weight'] = isset($input['max_weight']) ? wc_format_decimal($input['max_weight']) : 30;
        if ($output['max_weight'] <= $output['min_weight']) {
            $output['max_weight'] = $output['min_weight'] + 1;
        }
        
        // Enable volumetric weight
        $output['enable_volumetric_weight'] = isset($input['enable_volumetric_weight']) ? 'yes' : 'no';
        
        // Volumetric divisor
        $output['volumetric_divisor'] = isset($input['volumetric_divisor']) ? intval($input['volumetric_divisor']) : 5000;
        if ($output['volumetric_divisor'] <= 0) {
            $output['volumetric_divisor'] = 5000;
        }
        
        // COD fee percentage
        $output['cod_fee_percentage'] = isset($input['cod_fee_percentage']) ? wc_format_decimal($input['cod_fee_percentage']) : 10;
        if ($output['cod_fee_percentage'] < 0) {
            $output['cod_fee_percentage'] = 0;
        }
        
        // Courier products
        $output['courier_products'] = isset($input['courier_products']) ? array_map('intval', (array) $input['courier_products']) : array();
        
        // Include in emails
        $output['include_in_emails'] = isset($input['include_in_emails']) ? 'yes' : 'no';
        
        return $output;
    }
    
    /**
     * Validate display settings.
     *
     * @since    1.0.0
     * @param    array    $input    The input array to validate.
     * @return   array              The validated input array.
     */
    public function validate_display_settings($input) {
        $output = array();
        
        // Show weight details
        $output['show_weight_details'] = isset($input['show_weight_details']) ? (array) $input['show_weight_details'] : array();
        $valid_locations = array('cart', 'checkout', 'order', 'email');
        $output['show_weight_details'] = array_intersect($output['show_weight_details'], $valid_locations);
        
        return $output;
    }
    
    /**
     * Render the general section.
     *
     * @since    1.0.0
     */
    public function render_general_section() {
        echo '<p>' . esc_html__('Configure general settings for the CargoHub courier plugin.', 'cargohub') . '</p>';
    }
    
    /**
     * Render the product section.
     *
     * @since    1.0.0
     */
    public function render_product_section() {
        echo '<p>' . esc_html__('Configure which products should be treated as courier products.', 'cargohub') . '</p>';
    }
    
    /**
     * Render the email section.
     *
     * @since    1.0.0
     */
    public function render_email_section() {
        echo '<p>' . esc_html__('Configure email settings for courier orders.', 'cargohub') . '</p>';
    }
    
    /**
     * Render the display section.
     *
     * @since    1.0.0
     */
    public function render_display_section() {
        echo '<p>' . esc_html__('Configure where to display weight and dimension details.', 'cargohub') . '</p>';
    }
    
    /**
     * Render the weight unit field.
     *
     * @since    1.0.0
     */
    public function render_weight_unit_field() {
        $options = $this->get_general_settings();
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        ?>
        <select name="cargohub_general_settings[weight_unit]" id="weight_unit">
            <option value="kg" <?php selected($weight_unit, 'kg'); ?>><?php esc_html_e('Kilograms (kg)', 'cargohub'); ?></option>
            <option value="g" <?php selected($weight_unit, 'g'); ?>><?php esc_html_e('Grams (g)', 'cargohub'); ?></option>
            <option value="lb" <?php selected($weight_unit, 'lb'); ?>><?php esc_html_e('Pounds (lb)', 'cargohub'); ?></option>
            <option value="oz" <?php selected($weight_unit, 'oz'); ?>><?php esc_html_e('Ounces (oz)', 'cargohub'); ?></option>
        </select>
        <p class="description"><?php esc_html_e('Select the weight unit to use for courier products.', 'cargohub'); ?></p>
        <?php
    }
    
    /**
     * Render the dimension unit field.
     *
     * @since    1.0.0
     */
    public function render_dimension_unit_field() {
        $options = $this->get_general_settings();
        $dimension_unit = isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
        ?>
        <select name="cargohub_general_settings[dimension_unit]" id="dimension_unit">
            <option value="cm" <?php selected($dimension_unit, 'cm'); ?>><?php esc_html_e('Centimeters (cm)', 'cargohub'); ?></option>
            <option value="m" <?php selected($dimension_unit, 'm'); ?>><?php esc_html_e('Meters (m)', 'cargohub'); ?></option>
            <option value="mm" <?php selected($dimension_unit, 'mm'); ?>><?php esc_html_e('Millimeters (mm)', 'cargohub'); ?></option>
            <option value="in" <?php selected($dimension_unit, 'in'); ?>><?php esc_html_e('Inches (in)', 'cargohub'); ?></option>
            <option value="ft" <?php selected($dimension_unit, 'ft'); ?>><?php esc_html_e('Feet (ft)', 'cargohub'); ?></option>
        </select>
        <p class="description"><?php esc_html_e('Select the dimension unit to use for courier products.', 'cargohub'); ?></p>
        <?php
    }
    
    /**
     * Render the minimum weight field.
     *
     * @since    1.0.0
     */
    public function render_min_weight_field() {
        $options = $this->get_general_settings();
        $min_weight = isset($options['min_weight']) ? $options['min_weight'] : 0.5;
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        ?>
        <input type="number" step="0.01" min="0" name="cargohub_general_settings[min_weight]" id="min_weight" value="<?php echo esc_attr($min_weight); ?>" class="small-text" />
        <span><?php echo esc_html($weight_unit); ?></span>
        <p class="description"><?php esc_html_e('Minimum allowed weight for courier products.', 'cargohub'); ?></p>
        <?php
    }
    
    /**
     * Render the maximum weight field.
     *
     * @since    1.0.0
     */
    public function render_max_weight_field() {
        $options = $this->get_general_settings();
        $max_weight = isset($options['max_weight']) ? $options['max_weight'] : 30;
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        ?>
        <input type="number" step="0.01" min="0" name="cargohub_general_settings[max_weight]" id="max_weight" value="<?php echo esc_attr($max_weight); ?>" class="small-text" />
        <span><?php echo esc_html($weight_unit); ?></span>
        <p class="description"><?php esc_html_e('Maximum allowed weight for courier products.', 'cargohub'); ?></p>
        <?php
    }
    
    /**
     * Render the enable volumetric weight field.
     *
     * @since    1.0.0
     */
    public function render_enable_volumetric_weight_field() {
        $options = $this->get_general_settings();
        $enable_volumetric_weight = isset($options['enable_volumetric_weight']) ? $options['enable_volumetric_weight'] : 'yes';
        ?>
        <input type="checkbox" name="cargohub_general_settings[enable_volumetric_weight]" id="enable_volumetric_weight" value="yes" <?php checked($enable_volumetric_weight, 'yes'); ?> />
        <label for="enable_volumetric_weight"><?php esc_html_e('Use the greater of actual weight or volumetric weight for pricing', 'cargohub'); ?></label>
        <p class="description"><?php esc_html_e('Volumetric weight is calculated as (Length × Width × Height) ÷ Divisor.', 'cargohub'); ?></p>
        <?php
    }
    
    /**
     * Render the volumetric divisor field.
     *
     * @since    1.0.0
     */
    public function render_volumetric_divisor_field() {
        $options = $this->get_general_settings();
        $volumetric_divisor = isset($options['volumetric_divisor']) ? $options['volumetric_divisor'] : 5000;
        ?>
        <input type="number" step="1" min="1" name="cargohub_general_settings[volumetric_divisor]" id="volumetric_divisor" value="<?php echo esc_attr($volumetric_divisor); ?>" class="regular-text" />
        <p class="description">
            <?php esc_html_e('The divisor used in the volumetric weight formula. Common values:', 'cargohub'); ?><br />
            <?php esc_html_e('5000 - Common international standard (dimensions in cm, weight in kg)', 'cargohub'); ?><br />
            <?php esc_html_e('6000 - Used by some carriers for domestic shipments', 'cargohub'); ?><br />
            <?php esc_html_e('4000 - Used by some carriers for international air freight', 'cargohub'); ?>
        </p>
        <?php
    }
    
    /**
     * Render the COD fee percentage field.
     *
     * @since    1.0.0
     */
    public function render_cod_fee_percentage_field() {
        $options = $this->get_general_settings();
        $cod_fee_percentage = isset($options['cod_fee_percentage']) ? $options['cod_fee_percentage'] : 10;
        ?>
        <input type="number" step="0.01" min="0" name="cargohub_general_settings[cod_fee_percentage]" id="cod_fee_percentage" value="<?php echo esc_attr($cod_fee_percentage); ?>" class="small-text" />
        <span>%</span>
        <p class="description"><?php esc_html_e('Additional fee applied when COD (Cash on Delivery) is selected as a percentage of the shipping cost.', 'cargohub'); ?></p>
        <?php
    }
    
    /**
     * Render the courier products field.
     *
     * @since    1.0.0
     */
    public function render_courier_products_field() {
        $options = $this->get_general_settings();
        $courier_products = isset($options['courier_products']) ? $options['courier_products'] : array();
        
        // Get all products
        $args = array(
            'post_type' => 'product',
            'posts_per_page' => -1,
            'orderby' => 'title',
            'order' => 'ASC',
        );
        $products = get_posts($args);
        
        if (empty($products)) {
            echo '<p>' . esc_html__('No products found.', 'cargohub') . '</p>';
            return;
        }
        
        echo '<select name="cargohub_general_settings[courier_products][]" id="courier_products" multiple="multiple" style="width: 400px; height: 150px;">';
        
        foreach ($products as $product) {
            $product_id = $product->ID;
            $selected = in_array($product_id, $courier_products) ? 'selected="selected"' : '';
            echo '<option value="' . esc_attr($product_id) . '" ' . $selected . '>' . esc_html($product->post_title) . '</option>';
        }
        
        echo '</select>';
        echo '<p class="description">' . esc_html__('Select products that should be treated as courier products. Hold Ctrl/Cmd to select multiple products.', 'cargohub') . '</p>';
    }
    
    /**
     * Render the include in emails field.
     *
     * @since    1.0.0
     */
    public function render_include_in_emails_field() {
        $options = $this->get_general_settings();
        $include_in_emails = isset($options['include_in_emails']) ? $options['include_in_emails'] : 'yes';
        ?>
        <input type="checkbox" name="cargohub_general_settings[include_in_emails]" id="include_in_emails" value="yes" <?php checked($include_in_emails, 'yes'); ?> />
        <label for="include_in_emails"><?php esc_html_e('Include weight and dimension details in order emails', 'cargohub'); ?></label>
        <?php
    }
    
    /**
     * Render the show weight details field.
     *
     * @since    1.0.0
     */
    public function render_show_weight_details_field() {
        $options = get_option('cargohub_display_settings', array());
        $show_weight_details = isset($options['show_weight_details']) ? $options['show_weight_details'] : array('cart', 'checkout', 'order', 'email');
        ?>
        <fieldset>
            <legend class="screen-reader-text"><?php esc_html_e('Show weight details', 'cargohub'); ?></legend>
            
            <label for="show_weight_details_cart">
                <input type="checkbox" name="cargohub_display_settings[show_weight_details][]" id="show_weight_details_cart" value="cart" <?php checked(in_array('cart', $show_weight_details), true); ?> />
                <?php esc_html_e('Cart page', 'cargohub'); ?>
            </label><br />
            
            <label for="show_weight_details_checkout">
                <input type="checkbox" name="cargohub_display_settings[show_weight_details][]" id="show_weight_details_checkout" value="checkout" <?php checked(in_array('checkout', $show_weight_details), true); ?> />
                <?php esc_html_e('Checkout page', 'cargohub'); ?>
            </label><br />
            
            <label for="show_weight_details_order">
                <input type="checkbox" name="cargohub_display_settings[show_weight_details][]" id="show_weight_details_order" value="order" <?php checked(in_array('order', $show_weight_details), true); ?> />
                <?php esc_html_e('Order details page', 'cargohub'); ?>
            </label><br />
            
            <label for="show_weight_details_email">
                <input type="checkbox" name="cargohub_display_settings[show_weight_details][]" id="show_weight_details_email" value="email" <?php checked(in_array('email', $show_weight_details), true); ?> />
                <?php esc_html_e('Order emails', 'cargohub'); ?>
            </label>
        </fieldset>
        <?php
    }
    
    /**
     * Get the general settings.
     *
     * @since    1.0.0
     * @return   array    The general settings.
     */
    private function get_general_settings() {
        return get_option('cargohub_general_settings', array());
    }
}