<?php
/**
 * Admin View: Shipping Rates
 *
 * @package CargoHub
 */

// Exit if accessed directly.
if (!defined('WPINC')) {
    exit;
}

// Get settings
$options = get_option('cargohub_general_settings', array());
$weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';

// Get filter values
$filter_zone = isset($_GET['zone']) ? intval($_GET['zone']) : 0;
$filter_service = isset($_GET['service_type']) ? sanitize_text_field($_GET['service_type']) : '';

global $wpdb;

// Get zones for filter
$zones_table = $wpdb->prefix . 'cargohub_zones';
$zones = $wpdb->get_results("SELECT * FROM $zones_table ORDER BY zone_name ASC");

// Get service types for filter
$service_types_table = $wpdb->prefix . 'cargohub_service_types';
$service_types = $wpdb->get_results("
    SELECT * FROM $service_types_table
    WHERE active = 1
    ORDER BY service_name ASC
");

// Build query conditions based on filters
$where_clauses = array();
$where_args = array();

if ($filter_zone > 0) {
    $where_clauses[] = 'r.zone_id = %d';
    $where_args[] = $filter_zone;
}

if (!empty($filter_service)) {
    $where_clauses[] = 'r.service_type = %s';
    $where_args[] = $filter_service;
}

$where_sql = !empty($where_clauses) ? 'WHERE ' . implode(' AND ', $where_clauses) : '';

// Get rates
$rates_table = $wpdb->prefix . 'cargohub_rates';
$query = $wpdb->prepare(
    "SELECT r.*, z.zone_name, st.service_name
    FROM $rates_table r
    LEFT JOIN $zones_table z ON r.zone_id = z.id
    LEFT JOIN $service_types_table st ON r.service_type = st.service_code
    $where_sql
    ORDER BY r.zone_id, r.service_type, r.weight_from ASC",
    $where_args
);

$rates = $wpdb->get_results($query);

?>
<div class="wrap cargohub-rates">
    <h1 class="wp-heading-inline"><?php esc_html_e('Shipping Rates', 'cargohub'); ?></h1>
    <a href="#" class="page-title-action add-new-rate"><?php esc_html_e('Add New Rate', 'cargohub'); ?></a>
    <hr class="wp-header-end">
    
    <div id="ajax-response"></div>
    
    <div class="notice notice-info inline">
        <p><?php esc_html_e('Shipping rates define the cost of delivery based on weight ranges, service type, and shipping zone.', 'cargohub'); ?></p>
    </div>
    
    <!-- Filters -->
    <div class="tablenav top">
        <div class="alignleft actions">
            <form method="get">
                <input type="hidden" name="page" value="cargohub-rates">
                
                <select name="zone" id="filter-by-zone">
                    <option value=""><?php esc_html_e('All zones', 'cargohub'); ?></option>
                    <?php foreach ($zones as $zone) : ?>
                        <option value="<?php echo esc_attr($zone->id); ?>" <?php selected($filter_zone, $zone->id); ?>>
                            <?php echo esc_html($zone->zone_name); ?>
                        </option>
                    <?php endforeach; ?>
                </select>
                
                <select name="service_type" id="filter-by-service">
                    <option value=""><?php esc_html_e('All service types', 'cargohub'); ?></option>
                    <?php foreach ($service_types as $service_type) : ?>
                        <option value="<?php echo esc_attr($service_type->service_code); ?>" <?php selected($filter_service, $service_type->service_code); ?>>
                            <?php echo esc_html($service_type->service_name); ?>
                        </option>
                    <?php endforeach; ?>
                </select>
                
                <input type="submit" class="button" value="<?php esc_attr_e('Filter', 'cargohub'); ?>">
                <?php if ($filter_zone || $filter_service) : ?>
                    <a href="<?php echo esc_url(admin_url('admin.php?page=cargohub-rates')); ?>" class="button"><?php esc_html_e('Reset', 'cargohub'); ?></a>
                <?php endif; ?>
            </form>
        </div>
        <br class="clear">
    </div>
    
    <div id="col-container" class="wp-clearfix">
        <div id="col-left">
            <div class="col-wrap">
                <div class="form-wrap">
                    <h2><?php esc_html_e('Add New Shipping Rate', 'cargohub'); ?></h2>
                    <form id="add-rate" method="post">
                        <div class="form-field">
                            <label for="zone_id"><?php esc_html_e('Shipping Zone', 'cargohub'); ?></label>
                            <select name="zone_id" id="zone_id" required>
                                <option value=""><?php esc_html_e('-- Select Zone --', 'cargohub'); ?></option>
                                <?php foreach ($zones as $zone) : ?>
                                    <option value="<?php echo esc_attr($zone->id); ?>" <?php selected($filter_zone, $zone->id); ?>>
                                        <?php echo esc_html($zone->zone_name); ?>
                                    </option>
                                <?php endforeach; ?>
                            </select>
                            <p><?php esc_html_e('The zone where this rate applies.', 'cargohub'); ?></p>
                        </div>
                        
                        <div class="form-field">
                            <label for="service_type"><?php esc_html_e('Service Type', 'cargohub'); ?></label>
                            <select name="service_type" id="service_type" required>
                                <option value=""><?php esc_html_e('-- Select Service Type --', 'cargohub'); ?></option>
                                <?php foreach ($service_types as $service_type) : ?>
                                    <option value="<?php echo esc_attr($service_type->service_code); ?>" <?php selected($filter_service, $service_type->service_code); ?>>
                                        <?php echo esc_html($service_type->service_name); ?>
                                    </option>
                                <?php endforeach; ?>
                            </select>
                            <p><?php esc_html_e('The service type for this rate.', 'cargohub'); ?></p>
                        </div>
                        
                        <div class="form-field">
                            <label><?php esc_html_e('Weight Range', 'cargohub'); ?></label>
                            <div class="weight-range-inputs">
                                <input type="number" step="0.01" min="0" name="weight_from" id="weight_from" placeholder="<?php esc_attr_e('From', 'cargohub'); ?>" required style="width: 45%;">
                                <span class="weight-range-separator">—</span>
                                <input type="number" step="0.01" min="0" name="weight_to" id="weight_to" placeholder="<?php esc_attr_e('To', 'cargohub'); ?>" required style="width: 45%;">
                            </div>
                            <p>
                                <?php 
                                // Translators: %s is the weight unit (kg, lb, etc.)
                                printf(esc_html__('Weight range in %s. The rate applies when the weight is >= From and < To.', 'cargohub'), esc_html($weight_unit)); 
                                ?>
                            </p>
                        </div>
                        
                        <div class="form-field">
                            <label for="price"><?php esc_html_e('Price', 'cargohub'); ?></label>
                            <div class="price-input">
                                <span class="currency-symbol"><?php echo esc_html(get_woocommerce_currency_symbol()); ?></span>
                                <input type="number" step="0.01" min="0" name="price" id="price" required>
                            </div>
                            <p><?php esc_html_e('The shipping cost for this weight range.', 'cargohub'); ?></p>
                        </div>
                        
                        <p class="submit">
                            <input type="hidden" name="cargohub_action" value="save_rate">
                            <input type="hidden" name="id" value="">
                            <input type="hidden" name="nonce" value="<?php echo esc_attr(wp_create_nonce('cargohub-admin-nonce')); ?>">
                            <button type="submit" class="button button-primary" id="submit-rate"><?php esc_html_e('Add New Rate', 'cargohub'); ?></button>
                        </p>
                    </form>
                </div>
            </div>
        </div>
        
        <div id="col-right">
            <div class="col-wrap">
                <table class="widefat fixed striped" id="rate-table">
                    <thead>
                        <tr>
                            <th scope="col" class="manage-column column-zone"><?php esc_html_e('Zone', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-service"><?php esc_html_e('Service Type', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-weight"><?php esc_html_e('Weight Range', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-price"><?php esc_html_e('Price', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-actions"><?php esc_html_e('Actions', 'cargohub'); ?></th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php if (empty($rates)) : ?>
                            <tr class="no-items">
                                <td class="colspanchange" colspan="5"><?php esc_html_e('No rates found.', 'cargohub'); ?></td>
                            </tr>
                        <?php else : ?>
                            <?php foreach ($rates as $rate) : ?>
                                <tr data-id="<?php echo esc_attr($rate->id); ?>">
                                    <td class="column-zone">
                                        <?php echo esc_html($rate->zone_name); ?>
                                    </td>
                                    <td class="column-service">
                                        <?php echo esc_html($rate->service_name); ?>
                                    </td>
                                    <td class="column-weight">
                                        <?php 
                                        // Translators: %1$s is the starting weight, %2$s is the ending weight, %3$s is the weight unit
                                        printf(
                                            esc_html__('%1$s — %2$s %3$s', 'cargohub'),
                                            esc_html(wc_format_decimal($rate->weight_from, 2)),
                                            esc_html(wc_format_decimal($rate->weight_to, 2)),
                                            esc_html($weight_unit)
                                        ); 
                                        ?>
                                    </td>
                                    <td class="column-price">
                                        <?php echo wp_kses_post(wc_price($rate->price)); ?>
                                    </td>
                                    <td class="column-actions">
                                        <a href="#" class="edit-rate" 
                                           data-id="<?php echo esc_attr($rate->id); ?>"
                                           data-zone-id="<?php echo esc_attr($rate->zone_id); ?>"
                                           data-service-type="<?php echo esc_attr($rate->service_type); ?>"
                                           data-weight-from="<?php echo esc_attr($rate->weight_from); ?>"
                                           data-weight-to="<?php echo esc_attr($rate->weight_to); ?>"
                                           data-price="<?php echo esc_attr($rate->price); ?>">
                                            <?php esc_html_e('Edit', 'cargohub'); ?>
                                        </a> | 
                                        <a href="#" class="delete-rate" data-id="<?php echo esc_attr($rate->id); ?>">
                                            <?php esc_html_e('Delete', 'cargohub'); ?>
                                        </a>
                                    </td>
                                </tr>
                            <?php endforeach; ?>
                        <?php endif; ?>
                    </tbody>
                    <tfoot>
                        <tr>
                            <th scope="col" class="manage-column column-zone"><?php esc_html_e('Zone', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-service"><?php esc_html_e('Service Type', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-weight"><?php esc_html_e('Weight Range', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-price"><?php esc_html_e('Price', 'cargohub'); ?></th>
                            <th scope="col" class="manage-column column-actions"><?php esc_html_e('Actions', 'cargohub'); ?></th>
                        </tr>
                    </tfoot>
                </table>
            </div>
        </div>
    </div>
</div>

<script type="text/javascript">
jQuery(document).ready(function($) {
    // Handle form submission
    $('#add-rate').on('submit', function(e) {
        e.preventDefault();
        
        var form = $(this);
        var submitButton = $('#submit-rate');
        var weightFrom = parseFloat(form.find('input[name="weight_from"]').val());
        var weightTo = parseFloat(form.find('input[name="weight_to"]').val());
        
        // Validate weight range
        if (weightFrom >= weightTo) {
            alert('<?php esc_html_e('The "From" weight must be less than the "To" weight.', 'cargohub'); ?>');
            return;
        }
        
        var formData = {
            'cargohub_action': form.find('input[name="cargohub_action"]').val(),
            'id': form.find('input[name="id"]').val(),
            'zone_id': form.find('select[name="zone_id"]').val(),
            'service_type': form.find('select[name="service_type"]').val(),
            'weight_from': weightFrom,
            'weight_to': weightTo,
            'price': form.find('input[name="price"]').val(),
            'nonce': form.find('input[name="nonce"]').val(),
            'action': 'cargohub_admin_action'
        };
        
        // Disable submit button and show loading
        submitButton.prop('disabled', true).text(
            form.find('input[name="id"]').val() ? 
            '<?php esc_html_e('Updating...', 'cargohub'); ?>' : 
            '<?php esc_html_e('Adding...', 'cargohub'); ?>'
        );
        
        $.ajax({
            type: 'POST',
            url: ajaxurl,
            data: formData,
            success: function(response) {
                if (response.success) {
                    // Show success message
                    $('#ajax-response').html('<div class="notice notice-success is-dismissible"><p>' + response.data.message + '</p></div>');
                    
                    // Reset form if adding new
                    if (!form.find('input[name="id"]').val()) {
                        form.find('input[name="weight_from"], input[name="weight_to"], input[name="price"]').val('');
                    }
                    
                    // Reload the page to update the table
                    setTimeout(function() {
                        location.reload();
                    }, 1000);
                } else {
                    // Show error message
                    $('#ajax-response').html('<div class="notice notice-error is-dismissible"><p>' + response.data.message + '</p></div>');
                    
                    // Re-enable submit button
                    submitButton.prop('disabled', false).text(
                        form.find('input[name="id"]').val() ? 
                        '<?php esc_html_e('Update Rate', 'cargohub'); ?>' : 
                        '<?php esc_html_e('Add New Rate', 'cargohub'); ?>'
                    );
                }
            },
            error: function() {
                // Show error message
                $('#ajax-response').html('<div class="notice notice-error is-dismissible"><p><?php esc_html_e('An error occurred. Please try again.', 'cargohub'); ?></p></div>');
                
                // Re-enable submit button
                submitButton.prop('disabled', false).text(
                    form.find('input[name="id"]').val() ? 
                    '<?php esc_html_e('Update Rate', 'cargohub'); ?>' : 
                    '<?php esc_html_e('Add New Rate', 'cargohub'); ?>'
                );
            }
        });
    });
    
    // Handle edit click
    $('.edit-rate').on('click', function(e) {
        e.preventDefault();
        
        var id = $(this).data('id');
        var zoneId = $(this).data('zone-id');
        var serviceType = $(this).data('service-type');
        var weightFrom = $(this).data('weight-from');
        var weightTo = $(this).data('weight-to');
        var price = $(this).data('price');
        
        // Populate form
        $('#zone_id').val(zoneId);
        $('#service_type').val(serviceType);
        $('#weight_from').val(weightFrom);
        $('#weight_to').val(weightTo);
        $('#price').val(price);
        
        // Change form to edit mode
        $('input[name="id"]').val(id);
        $('#submit-rate').text('<?php esc_html_e('Update Rate', 'cargohub'); ?>');
        $('h2:first').text('<?php esc_html_e('Edit Shipping Rate', 'cargohub'); ?>');
        
        // Scroll to form
        $('html, body').animate({
            scrollTop: $('#add-rate').offset().top - 50
        }, 500);
    });
    
    // Handle delete click
    $('.delete-rate').on('click', function(e) {
        e.preventDefault();
        
        var id = $(this).data('id');
        
        if (confirm('<?php esc_html_e('Are you sure you want to delete this shipping rate? This action cannot be undone.', 'cargohub'); ?>')) {
            $.ajax({
                type: 'POST',
                url: ajaxurl,
                data: {
                    'action': 'cargohub_admin_action',
                    'cargohub_action': 'delete_rate',
                    'id': id,
                    'nonce': '<?php echo wp_create_nonce('cargohub-admin-nonce'); ?>'
                },
                success: function(response) {
                    if (response.success) {
                        // Show success message
                        $('#ajax-response').html('<div class="notice notice-success is-dismissible"><p>' + response.data.message + '</p></div>');
                        
                        // Remove row from table
                        $('tr[data-id="' + id + '"]').fadeOut(400, function() {
                            $(this).remove();
                            
                            // Check if table is empty
                            if ($('#rate-table tbody tr').length === 0) {
                                $('#rate-table tbody').html('<tr class="no-items"><td class="colspanchange" colspan="5"><?php esc_html_e('No rates found.', 'cargohub'); ?></td></tr>');
                            }
                        });
                    } else {
                        // Show error message
                        $('#ajax-response').html('<div class="notice notice-error is-dismissible"><p>' + response.data.message + '</p></div>');
                    }
                },
                error: function() {
                    // Show error message
                    $('#ajax-response').html('<div class="notice notice-error is-dismissible"><p><?php esc_html_e('An error occurred. Please try again.', 'cargohub'); ?></p></div>');
                }
            });
        }
    });
    
    // Handle add new click
    $('.add-new-rate').on('click', function(e) {
        e.preventDefault();
        
        // Reset form
        $('#add-rate')[0].reset();
        $('input[name="id"]').val('');
        $('#submit-rate').text('<?php esc_html_e('Add New Rate', 'cargohub'); ?>');
        $('h2:first').text('<?php esc_html_e('Add New Shipping Rate', 'cargohub'); ?>');
        
        // Scroll to form
        $('html, body').animate({
            scrollTop: $('#add-rate').offset().top - 50
        }, 500);
    });
});
</script>

<style>
.weight-range-inputs {
    display: flex;
    align-items: center;
}
.weight-range-separator {
    margin: 0 10px;
}
.price-input {
    position: relative;
}
.price-input .currency-symbol {
    position: absolute;
    top: 8px;
    left: 10px;
}
.price-input input {
    padding-left: 25px;
}
.column-zone, .column-service {
    width: 25%;
}
.column-weight, .column-price {
    width: 20%;
}
.column-actions {
    width: 10%;
}
</style>