<?php
/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 */
class CargoHub_Activator {

    /**
     * Initialize plugin settings and create necessary database tables.
     *
     * @since    1.0.0
     */
    public static function activate() {
        global $wpdb;
        
        // Create database tables if needed
        self::create_tables();
        
        // Initialize default settings
        self::init_settings();
        
        // Set up initial service types
        self::init_service_types();
        
        // Force WooCommerce to refresh permalink rules
        update_option('woocommerce_queue_flush_rewrite_rules', 'yes');
        
        // Store activation time
        if (!get_option('cargohub_activation_time')) {
            update_option('cargohub_activation_time', time());
        }
    }
    
    /**
     * Create custom database tables.
     *
     * @since    1.0.0
     */
    private static function create_tables() {
        global $wpdb;
        
        $charset_collate = $wpdb->get_charset_collate();
        
        // Table for service rates
        $table_name = $wpdb->prefix . 'cargohub_rates';
        
        $sql = "CREATE TABLE $table_name (
            id bigint(20) NOT NULL AUTO_INCREMENT,
            weight_from decimal(10,2) NOT NULL,
            weight_to decimal(10,2) NOT NULL,
            service_type varchar(50) NOT NULL,
            price decimal(10,2) NOT NULL,
            zone_id bigint(20) NOT NULL DEFAULT 0,
            created_at datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            PRIMARY KEY  (id)
        ) $charset_collate;";
        
        // Table for service zones
        $table_name_zones = $wpdb->prefix . 'cargohub_zones';
        
        $sql .= "CREATE TABLE $table_name_zones (
            id bigint(20) NOT NULL AUTO_INCREMENT,
            zone_name varchar(100) NOT NULL,
            countries text NOT NULL,
            created_at datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            PRIMARY KEY  (id)
        ) $charset_collate;";
        
        // Table for service types
        $table_name_service_types = $wpdb->prefix . 'cargohub_service_types';
        
        $sql .= "CREATE TABLE $table_name_service_types (
            id bigint(20) NOT NULL AUTO_INCREMENT,
            service_name varchar(100) NOT NULL,
            service_code varchar(50) NOT NULL,
            description text,
            active tinyint(1) NOT NULL DEFAULT 1,
            created_at datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            PRIMARY KEY  (id)
        ) $charset_collate;";
        
        // Execute SQL to create tables
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }
    
    /**
     * Initialize default plugin settings.
     *
     * @since    1.0.0
     */
    private static function init_settings() {
        // General settings
        $general_settings = array(
            'cod_fee_percentage' => 10,
            'volumetric_divisor' => 5000, // Industry standard for courier services
            'enable_volumetric_weight' => 'yes',
            'min_weight' => 0.5,
            'max_weight' => 30,
            'weight_unit' => 'kg',
            'dimension_unit' => 'cm',
        );
        
        // Check if settings already exist, if not, add them
        if (!get_option('cargohub_general_settings')) {
            update_option('cargohub_general_settings', $general_settings);
        }
    }
    
    /**
     * Initialize default service types.
     *
     * @since    1.0.0
     */
    private static function init_service_types() {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        
        // Check if service types already exist
        $count = $wpdb->get_var("SELECT COUNT(*) FROM $table_name");
        
        if ($count == 0) {
            // Insert default service types
            $default_service_types = array(
                array(
                    'service_name' => 'Standard Delivery',
                    'service_code' => 'standard',
                    'description' => 'Regular delivery service (3-5 business days)',
                    'active' => 1
                ),
                array(
                    'service_name' => 'Express Delivery',
                    'service_code' => 'express',
                    'description' => 'Fast delivery service (1-2 business days)',
                    'active' => 1
                )
            );
            
            foreach ($default_service_types as $service_type) {
                $wpdb->insert(
                    $table_name,
                    $service_type
                );
            }
        }
        
        // Also set up a default zone if none exists
        $table_name_zones = $wpdb->prefix . 'cargohub_zones';
        $zone_count = $wpdb->get_var("SELECT COUNT(*) FROM $table_name_zones");
        
        if ($zone_count == 0) {
            $wpdb->insert(
                $table_name_zones,
                array(
                    'zone_name' => 'Domestic',
                    'countries' => serialize(array(WC()->countries->get_base_country()))
                )
            );
        }
        
        // Insert some sample rates if the rates table is empty
        $table_name_rates = $wpdb->prefix . 'cargohub_rates';
        $rates_count = $wpdb->get_var("SELECT COUNT(*) FROM $table_name_rates");
        
        if ($rates_count == 0) {
            $sample_rates = array(
                // Standard delivery rates
                array('weight_from' => 0, 'weight_to' => 1, 'service_type' => 'standard', 'price' => 10.00, 'zone_id' => 1),
                array('weight_from' => 1, 'weight_to' => 3, 'service_type' => 'standard', 'price' => 15.00, 'zone_id' => 1),
                array('weight_from' => 3, 'weight_to' => 5, 'service_type' => 'standard', 'price' => 20.00, 'zone_id' => 1),
                array('weight_from' => 5, 'weight_to' => 10, 'service_type' => 'standard', 'price' => 30.00, 'zone_id' => 1),
                array('weight_from' => 10, 'weight_to' => 20, 'service_type' => 'standard', 'price' => 45.00, 'zone_id' => 1),
                array('weight_from' => 20, 'weight_to' => 30, 'service_type' => 'standard', 'price' => 60.00, 'zone_id' => 1),
                
                // Express delivery rates
                array('weight_from' => 0, 'weight_to' => 1, 'service_type' => 'express', 'price' => 20.00, 'zone_id' => 1),
                array('weight_from' => 1, 'weight_to' => 3, 'service_type' => 'express', 'price' => 30.00, 'zone_id' => 1),
                array('weight_from' => 3, 'weight_to' => 5, 'service_type' => 'express', 'price' => 40.00, 'zone_id' => 1),
                array('weight_from' => 5, 'weight_to' => 10, 'service_type' => 'express', 'price' => 60.00, 'zone_id' => 1),
                array('weight_from' => 10, 'weight_to' => 20, 'service_type' => 'express', 'price' => 90.00, 'zone_id' => 1),
                array('weight_from' => 20, 'weight_to' => 30, 'service_type' => 'express', 'price' => 120.00, 'zone_id' => 1),
            );
            
            foreach ($sample_rates as $rate) {
                $wpdb->insert($table_name_rates, $rate);
            }
        }
    }
}