<?php
/**
 * The cart-specific functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The cart-specific functionality of the plugin.
 *
 * Handles all custom cart modifications for courier products.
 */
class CargoHub_Cart {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param    string    $plugin_name       The name of this plugin.
     * @param    string    $version           The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Customize the cart item price display.
     *
     * @since    1.0.0
     * @param    string    $price_html    The price HTML.
     * @param    array     $cart_item     The cart item data.
     * @param    string    $cart_item_key The cart item key.
     * @return   string                   The modified price HTML.
     */
    public function custom_cart_item_price($price_html, $cart_item, $cart_item_key) {
        // Check if this is a courier product with custom pricing
        if (isset($cart_item['cargohub_total_cost'])) {
            return wc_price($cart_item['cargohub_total_cost']);
        }
        
        return $price_html;
    }

    /**
     * Customize the cart item subtotal display.
     *
     * @since    1.0.0
     * @param    string    $subtotal_html The subtotal HTML.
     * @param    array     $cart_item     The cart item data.
     * @param    string    $cart_item_key The cart item key.
     * @return   string                   The modified subtotal HTML.
     */
    public function custom_cart_item_subtotal($subtotal_html, $cart_item, $cart_item_key) {
        // Check if this is a courier product with custom pricing
        if (isset($cart_item['cargohub_total_cost'])) {
            return wc_price($cart_item['cargohub_total_cost']);
        }
        
        return $subtotal_html;
    }

    /**
     * Adjust the product price for courier products.
     *
     * @since    1.0.0
     * @param    float    $price      The original price.
     * @param    object   $product    The product object.
     * @return   float                The adjusted price.
     */
    public function adjust_cart_item_price($price, $product) {
        // Get cart items
        if (WC()->cart) {
            foreach (WC()->cart->get_cart() as $cart_item) {
                // Check if this is the product in the cart and it has custom pricing
                if ($product->get_id() == $cart_item['product_id'] && isset($cart_item['cargohub_total_cost'])) {
                    return $cart_item['cargohub_total_cost'];
                }
            }
        }
        
        return $price;
    }

    /**
     * Adjust the cart item data before calculating totals.
     *
     * @since    1.0.0
     * @param    array    $cart_item_data    The cart item data.
     * @param    string   $cart_item_key     The cart item key.
     * @return   array                       The modified cart item data.
     */
    public function adjust_cart_item_data($cart_item_data, $cart_item_key) {
        // Check if this is a courier product with custom pricing
        if (isset($cart_item_data['cargohub_total_cost'])) {
            // Set the line_total and line_subtotal to our custom price
            $cart_item_data['data']->set_price($cart_item_data['cargohub_total_cost']);
        }
        
        return $cart_item_data;
    }

    /**
     * Override the cart item price when adding to cart.
     *
     * @since    1.0.0
     * @param    array    $cart_contents    The cart contents.
     * @return   array                      The modified cart contents.
     */
    public function before_calculate_totals($cart_contents) {
        if (is_admin() && !defined('DOING_AJAX')) {
            return $cart_contents;
        }

        foreach ($cart_contents as $cart_item_key => $cart_item) {
            if (isset($cart_item['cargohub_total_cost'])) {
                $cart_item['data']->set_price($cart_item['cargohub_total_cost']);
            }
        }
        
        return $cart_contents;
    }

    /**
     * Add dimension information to the cart item.
     *
     * @since    1.0.0
     * @param    array    $cart_item     The cart item data.
     * @param    string   $cart_item_key The cart item key.
     */
    public function add_dimension_info_to_cart($cart_item, $cart_item_key) {
        // Check if this is a courier product with dimension data
        if (!isset($cart_item['cargohub_weight']) || !isset($cart_item['cargohub_service_type'])) {
            return;
        }
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        $dimension_unit = isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
        
        // Get display settings
        $display_options = get_option('cargohub_display_settings', array());
        $show_weight_details = isset($display_options['show_weight_details']) ? $display_options['show_weight_details'] : array('cart');
        
        // If we shouldn't show details in cart, return
        if (!in_array('cart', $show_weight_details)) {
            return;
        }
        
        // Get service type name
        $service_type_name = $this->get_service_type_name($cart_item['cargohub_service_type']);
        
        // Start output
        echo '<div class="cargohub-cart-item-details">';
        
        // Show service type and COD badge
        echo '<p>';
        echo '<span class="cargohub-service-badge">' . esc_html($service_type_name) . '</span>';
        
        if (isset($cart_item['cargohub_cod_option']) && $cart_item['cargohub_cod_option'] === 'yes') {
            echo '<span class="cargohub-cod-badge">' . esc_html__('COD', 'cargohub') . '</span>';
        }
        echo '</p>';
        
        // Show weight and dimensions
        echo '<p>';
        
        // Weight
        echo esc_html__('Weight', 'cargohub') . ': ' . esc_html(wc_format_decimal($cart_item['cargohub_weight'], 2)) . ' ' . esc_html($weight_unit);
        
        // Show volumetric weight if relevant
        if (isset($cart_item['cargohub_volumetric_weight']) && $cart_item['cargohub_volumetric_weight'] > $cart_item['cargohub_weight']) {
            echo ' (' . esc_html__('Volumetric', 'cargohub') . ': ' . esc_html(wc_format_decimal($cart_item['cargohub_volumetric_weight'], 2)) . ' ' . esc_html($weight_unit) . ')';
        }
        
        echo '</p>';
        
        // Dimensions
        if (isset($cart_item['cargohub_length']) && isset($cart_item['cargohub_width']) && isset($cart_item['cargohub_height'])) {
            echo '<p>';
            echo esc_html__('Dimensions', 'cargohub') . ': ';
            echo esc_html(wc_format_decimal($cart_item['cargohub_length'], 1)) . ' × ';
            echo esc_html(wc_format_decimal($cart_item['cargohub_width'], 1)) . ' × ';
            echo esc_html(wc_format_decimal($cart_item['cargohub_height'], 1)) . ' ' . esc_html($dimension_unit);
            echo '</p>';
        }
        
        // End output
        echo '</div>';
    }

    /**
     * Hide the quantity input field for courier products.
     *
     * @since    1.0.0
     * @param    string   $product_quantity The quantity field HTML.
     * @param    string   $cart_item_key    The cart item key.
     * @param    array    $cart_item        The cart item data.
     * @return   string                     The modified quantity field HTML.
     */
    public function hide_quantity_input($product_quantity, $cart_item_key, $cart_item) {
        // Check if this is a courier product
        if (isset($cart_item['cargohub_weight']) && isset($cart_item['cargohub_service_type'])) {
            return '<div class="quantity"><input type="hidden" name="cart[' . $cart_item_key . '][qty]" value="1" /></div>';
        }
        
        return $product_quantity;
    }

    /**
     * Prevent quantity updates for courier products.
     *
     * @since    1.0.0
     * @param    bool    $cart_updated Whether the cart was updated.
     * @return   bool                   Whether the cart was updated.
     */
    public function prevent_quantity_update($cart_updated) {
        // Ensure courier products always have a quantity of 1
        foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {
            if (isset($cart_item['cargohub_weight']) && isset($cart_item['cargohub_service_type'])) {
                WC()->cart->set_quantity($cart_item_key, 1, true);
            }
        }
        
        return $cart_updated;
    }

    /**
     * Get the service type name based on service code.
     *
     * @since    1.0.0
     * @param    string    $service_code    The service type code.
     * @return   string                     The service type name.
     */
    private function get_service_type_name($service_code) {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $service_name = $wpdb->get_var($wpdb->prepare(
            "SELECT service_name FROM $table_name WHERE service_code = %s",
            $service_code
        ));
        
        if ($service_name) {
            return $service_name;
        }
        
        return $service_code;
    }
}