<?php
/**
 * The checkout-specific functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The checkout-specific functionality of the plugin.
 *
 * Handles all custom checkout modifications for courier products.
 */
class CargoHub_Checkout {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param    string    $plugin_name       The name of this plugin.
     * @param    string    $version           The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        
        // Add filter to change "Ship to a different address?" label
        add_filter('woocommerce_ship_to_different_address_text', array($this, 'change_ship_to_different_address_label'));
        
        // Add global text replacement filter
        add_filter('gettext', array($this, 'shipchange'), 20, 3);
        
        // Add hook for displaying courier info
        add_action('woocommerce_checkout_before_customer_details', array($this, 'display_courier_info_on_checkout'));
        
        // Add styles and scripts
        add_action('wp_head', array($this, 'add_checkout_styles'));
        add_action('wp_footer', array($this, 'add_checkout_scripts'));
        
        // Add bank details fields when COD is selected
        add_action('woocommerce_after_checkout_billing_form', array($this, 'add_bank_details_fields'));
    }

    /**
     * Change the "Ship to a different address?" label to "Shipping Address".
     *
     * @since    1.0.0
     * @return   string    The modified label.
     */
    public function change_ship_to_different_address_label() {
        return __('Consignee address', 'woocommerce');
    }

    /**
     * Change "Ship to a different address?" text globally.
     *
     * @since    1.0.0
     * @param    string    $translated_text    The translated text.
     * @param    string    $text               The original text.
     * @param    string    $domain             The text domain.
     * @return   string                        The modified text.
     */
    public function shipchange($translated_text, $text, $domain) {
        switch ($translated_text) {
            case 'Ship to a different address?':
                $translated_text = __('Consignee address', 'woocommerce');
                break;
        }
        return $translated_text;
    }

    /**
     * Customize the checkout fields.
     *
     * @since    1.0.0
     * @param    array    $fields    The checkout fields.
     * @return   array               The modified checkout fields.
     */
    public function customize_checkout_fields($fields) {
        // Check if there are courier products in the cart
        if (!$this->has_courier_products_in_cart()) {
            return $fields;
        }
        
        // Make shipping address fields required
        if (isset($fields['shipping'])) {
            foreach ($fields['shipping'] as $key => $field) {
                // Skip fields that shouldn't be required
                if (in_array($key, array('shipping_company', 'shipping_address_2', 'shipping_state'))) {
                    continue;
                }
                
                $fields['shipping'][$key]['required'] = true;
            }
        }
        
        // Remove order notes
        if (isset($fields['order']) && isset($fields['order']['order_comments'])) {
         unset($fields['order']['order_comments']);
        }
        
        // Add custom fields for receiver contact details
        if (isset($fields['shipping'])) {
            // Add receiver phone
            $fields['shipping']['shipping_phone'] = array(
                'label'       => __('Receiver Phone', 'cargohub'),
                'placeholder' => __('Phone number for delivery', 'cargohub'),
                'required'    => true,
                'class'       => array('form-row-wide'),
                'clear'       => true,
                'priority'    => 25
            );
            
            // Add delivery notes field
            $fields['shipping']['shipping_notes'] = array(
                'label'       => __('Delivery Instructions', 'cargohub'),
                'placeholder' => __('Special instructions for delivery (optional)', 'cargohub'),
                'required'    => false,
                'class'       => array('form-row-wide'),
                'clear'       => true,
                'type'        => 'textarea',
                'priority'    => 120
            );
        }
        
        return $fields;
    }

    /**
     * Change the shipping fields heading.
     *
     * @since    1.0.0
     * @param    string    $heading    The original heading.
     * @return   string                The modified heading.
     */
    public function change_shipping_fields_heading($heading) {
        // Check if there are courier products in the cart
        if (!$this->has_courier_products_in_cart()) {
            return $heading;
        }
        
        return __('Shipping Address', 'cargohub');
    }

    /**
     * Add bank details fields to the checkout form when COD is selected.
     *
     * @since    1.0.0
     * @param    WC_Checkout $checkout The checkout object.
     */
    public function add_bank_details_fields($checkout) {
        // Check if there are courier products in the cart
        if (!$this->has_courier_products_in_cart()) {
            return;
        }
        
        echo '<div id="cargohub_bank_details" style="margin-top: 30px; clear: both;">';
        echo '<h3>' . esc_html__('Bank Account Details(For COD Orders)', 'cargohub') . '</h3>';
        
        // Account holder name field
        woocommerce_form_field('bank_account_name', array(
            'type'          => 'text',
            'class'         => array('form-row-wide', 'cargohub-bank-field'),
            'label'         => __('Account Holder Name', 'cargohub'),
            'placeholder'   => __('Enter account holder name', 'cargohub'),
            'required'      => false, // Not required by default, JS handles validation
        ), $checkout->get_value('bank_account_name'));
        
        // Account number field
        woocommerce_form_field('bank_account_number', array(
            'type'          => 'text',
            'class'         => array('form-row-wide', 'cargohub-bank-field'),
            'label'         => __('Account Number', 'cargohub'),
            'placeholder'   => __('Enter account number', 'cargohub'),
            'required'      => false, // Not required by default, JS handles validation
        ), $checkout->get_value('bank_account_number'));
        
        // IFSC code field
        woocommerce_form_field('bank_ifsc_code', array(
            'type'          => 'text',
            'class'         => array('form-row-wide', 'cargohub-bank-field'),
            'label'         => __('IFSC Code', 'cargohub'),
            'placeholder'   => __('Enter IFSC code', 'cargohub'),
            'required'      => false, // Not required by default, JS handles validation
        ), $checkout->get_value('bank_ifsc_code'));
        
        echo '</div>';
        
        // Add hidden fields to store COD status
        echo '<input type="hidden" id="cargohub_is_cod" name="cargohub_is_cod" value="0">';
        
        // Add custom validation script
        ?>
        <script type="text/javascript">
        jQuery(function($) {
            // Add validation on checkout submission
            $('form.checkout').on('checkout_place_order', function() {
                var paymentMethod = $('input[name="payment_method"]:checked').val();
                var codBadges = $('.cargohub-cod-badge').length;
                var isCOD = paymentMethod === 'cod' || codBadges > 0;
                
                // Update the hidden field
                $('#cargohub_is_cod').val(isCOD ? '1' : '0');
                
                // Only validate bank fields if COD is selected
                if (isCOD) {
                    var accountName = $('#bank_account_name').val();
                    var accountNumber = $('#bank_account_number').val();
                    var ifscCode = $('#bank_ifsc_code').val();
                    
                    var isValid = true;
                    
                    // Clear previous error messages
                    $('.woocommerce-error').remove();
                    
                    // Validate account name
                    if (!accountName || accountName.trim() === '') {
                        isValid = false;
                        $('#bank_account_name').addClass('woocommerce-invalid');
                    } else {
                        $('#bank_account_name').removeClass('woocommerce-invalid');
                    }
                    
                    // Validate account number
                    if (!accountNumber || accountNumber.trim() === '') {
                        isValid = false;
                        $('#bank_account_number').addClass('woocommerce-invalid');
                    } else {
                        $('#bank_account_number').removeClass('woocommerce-invalid');
                    }
                    
                    // Validate IFSC code
                    if (!ifscCode || ifscCode.trim() === '') {
                        isValid = false;
                        $('#bank_ifsc_code').addClass('woocommerce-invalid');
                    } else {
                        $('#bank_ifsc_code').removeClass('woocommerce-invalid');
                    }
                    
                    // If validation fails, show an error message
                    if (!isValid) {
                        $('form.checkout').prepend('<div class="woocommerce-error">Please fill in all bank details fields.</div>');
                        $('html, body').animate({
                            scrollTop: $('.woocommerce-error').offset().top - 100
                        }, 500);
                        return false;
                    }
                }
                
                return true;
            });
        });
        </script>
        <?php
    }

    /**
     * Save custom checkout fields to the order.
     *
     * @since    1.0.0
     * @param    int    $order_id    The order ID.
     */
    public function save_custom_checkout_fields($order_id) {
        // Check if this is a COD order
        $is_cod = false;
        
        // Get the payment method
        $order = wc_get_order($order_id);
        if ($order && $order->get_payment_method() === 'cod') {
            $is_cod = true;
        }
        
        // Check the hidden field value
        if (isset($_POST['cargohub_is_cod']) && $_POST['cargohub_is_cod'] === '1') {
            $is_cod = true;
        }
        
        // Save bank details fields if this is a COD order
        if ($is_cod) {
            if (!empty($_POST['bank_account_name'])) {
                update_post_meta($order_id, '_bank_account_name', sanitize_text_field($_POST['bank_account_name']));
            }
            
            if (!empty($_POST['bank_account_number'])) {
                update_post_meta($order_id, '_bank_account_number', sanitize_text_field($_POST['bank_account_number']));
            }
            
            if (!empty($_POST['bank_ifsc_code'])) {
                update_post_meta($order_id, '_bank_ifsc_code', sanitize_text_field($_POST['bank_ifsc_code']));
            }
        }
        
        // Save receiver phone
        if (isset($_POST['shipping_phone']) && !empty($_POST['shipping_phone'])) {
            update_post_meta($order_id, '_shipping_phone', sanitize_text_field($_POST['shipping_phone']));
        }
        
        // Save delivery notes
        if (isset($_POST['shipping_notes']) && !empty($_POST['shipping_notes'])) {
            update_post_meta($order_id, '_shipping_notes', sanitize_textarea_field($_POST['shipping_notes']));
        }
    }

    /**
     * Display courier information on the checkout page.
     *
     * @since    1.0.0
     */
    public function display_courier_info_on_checkout() {
        // Check if there are courier products in the cart
        if (!$this->has_courier_products_in_cart()) {
            return;
        }
        
        // Get cart items
        $cart_items = WC()->cart->get_cart();
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        $dimension_unit = isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
        
        // Get display settings
        $display_options = get_option('cargohub_display_settings', array());
        $show_weight_details = isset($display_options['show_weight_details']) ? $display_options['show_weight_details'] : array('checkout');
        
        // If we shouldn't show details on checkout, return
        if (!in_array('checkout', $show_weight_details)) {
            return;
        }
        
        echo '<div class="cargohub-checkout-info">';
        echo '<h3>' . esc_html__('Shipping Details', 'cargohub') . '</h3>';
        
        echo '<table class="cargohub-checkout-info-table">';
        echo '<thead>';
        echo '<tr>';
        echo '<th>' . esc_html__('Service', 'cargohub') . '</th>';
        echo '<th>' . esc_html__('Weight', 'cargohub') . '</th>';
        echo '<th>' . esc_html__('Dimensions', 'cargohub') . '</th>';
        echo '<th>' . esc_html__('Options', 'cargohub') . '</th>';
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        
        foreach ($cart_items as $cart_item) {
            // Check if this is a courier product
            if (!isset($cart_item['cargohub_weight']) || !isset($cart_item['cargohub_service_type'])) {
                continue;
            }
            
            echo '<tr>';
            
            // Service type
            echo '<td>';
            $service_type_name = $this->get_service_type_name($cart_item['cargohub_service_type']);
            echo esc_html($service_type_name);
            echo '</td>';
            
            // Weight
            echo '<td>';
            echo esc_html(wc_format_decimal($cart_item['cargohub_weight'], 2)) . ' ' . esc_html($weight_unit);
            
            // Show volumetric weight if relevant
            if (isset($cart_item['cargohub_volumetric_weight']) && $cart_item['cargohub_volumetric_weight'] > $cart_item['cargohub_weight']) {
                echo '<br><small>' . esc_html__('Volumetric', 'cargohub') . ': ' . esc_html(wc_format_decimal($cart_item['cargohub_volumetric_weight'], 2)) . ' ' . esc_html($weight_unit) . '</small>';
            }
            echo '</td>';
            
            // Dimensions
            echo '<td>';
            if (isset($cart_item['cargohub_length']) && isset($cart_item['cargohub_width']) && isset($cart_item['cargohub_height'])) {
                echo esc_html(wc_format_decimal($cart_item['cargohub_length'], 1)) . ' × ';
                echo esc_html(wc_format_decimal($cart_item['cargohub_width'], 1)) . ' × ';
                echo esc_html(wc_format_decimal($cart_item['cargohub_height'], 1)) . ' ' . esc_html($dimension_unit);
            } else {
                echo '—';
            }
            echo '</td>';
            
            // Options
            echo '<td>';
            if (isset($cart_item['cargohub_cod_option']) && $cart_item['cargohub_cod_option'] === 'yes') {
                echo '<span class="cargohub-cod-badge">' . esc_html__('COD', 'cargohub') . '</span>';
            } else {
                echo '—';
            }
            echo '</td>';
            
            echo '</tr>';
        }
        
        echo '</tbody>';
        echo '</table>';
        echo '</div>';
    }

    /**
     * Add scripts for dynamic checkout behavior.
     *
     * @since    1.0.0
     */
    public function add_checkout_scripts() {
        // Only add on checkout page
        if (!is_checkout()) {
            return;
        }
        
        ?>
        <script type="text/javascript">
        jQuery(function($) {
            // Function to show/hide bank details based on payment method
            function toggleBankDetails() {
                var paymentMethod = $('input[name="payment_method"]:checked').val();
                var codBadges = $('.cargohub-cod-badge').length;
                var isCOD = paymentMethod === 'cod' || codBadges > 0;
                
                if (isCOD) {
                    $('#cargohub_bank_details').show();
                } else {
                    $('#cargohub_bank_details').hide();
                }
                
                // Update the hidden field
                $('#cargohub_is_cod').val(isCOD ? '1' : '0');
            }
            
            // Run on page load
            toggleBankDetails();
            
            // Run when payment method changes
            $(document.body).on('change', 'input[name="payment_method"]', function() {
                toggleBankDetails();
            });
            
            // Run after checkout is updated via AJAX
            $(document.body).on('updated_checkout', function() {
                setTimeout(toggleBankDetails, 500);
            });
            
            // Change "Ship to a different address?" text to "Shipping Address"
            function changeShippingText() {
                var $shippingCheckbox = $('#ship-to-different-address-checkbox');
                if ($shippingCheckbox.length > 0) {
                    $shippingCheckbox.siblings('label').text('<?php echo esc_js(__('Consignee address', 'woocommerce')); ?>');
                }
            }
            
            // Run on page load
            changeShippingText();
            
            // Also run after checkout is updated via AJAX
            $(document.body).on('updated_checkout', function() {
                changeShippingText();
            });
        });
        </script>
        <?php
    }

    /**
     * Add styles to enhance the checkout page.
     *
     * @since    1.0.0
     */
    public function add_checkout_styles() {
        // Only add on checkout page
        if (!is_checkout()) {
            return;
        }
        
        ?>
        <style type="text/css">
        /* Beautiful checkout styling */
        .woocommerce-checkout h3 {
            font-size: 24px;
            margin-bottom: 20px;
            padding-bottom: 10px;
            border-bottom: 2px solid #f8f9fa;
            color: #333;
        }
        
        /* Bank details styles */
        #cargohub_bank_details {
            background: #fff;
            padding: 25px;
            border-radius: 8px;
            box-shadow: 0 2px 10px rgba(0,0,0,0.05);
            margin-bottom: 30px;
            border-left: 4px solid #ff6600;
        }
        
        .cargohub-bank-field {
            background-color: #fafafa;
            padding: 10px;
            margin-bottom: 15px;
        }
        
        /* Error highlighting for bank fields */
        .woocommerce-invalid .cargohub-bank-field {
            border-color: #e2401c;
        }
        
        /* COD badge */
        .cargohub-cod-badge {
            display: inline-block;
            padding: 3px 8px;
            background-color: #ffe5e5;
            color: #d63638;
            border-radius: 3px;
            font-size: 12px;
            font-weight: 600;
        }
        
        /* Shipping details table */
        .cargohub-checkout-info {
            margin-bottom: 30px;
        }
        
        .cargohub-checkout-info h3 {
            font-size: 20px;
            margin-bottom: 15px;
        }
        
        .cargohub-checkout-info-table {
            width: 100%;
            border-collapse: collapse;
            background: #fff;
            border-radius: 8px;
            overflow: hidden;
            box-shadow: 0 2px 10px rgba(0,0,0,0.05);
        }
        
        .cargohub-checkout-info-table th {
            background-color: #f8f9fa;
            padding: 12px 15px;
            text-align: left;
            font-weight: 600;
            color: #333;
        }
        
        .cargohub-checkout-info-table td {
            padding: 12px 15px;
            border-top: 1px solid #f1f1f1;
        }
        
        /* Responsive styling */
        @media (max-width: 768px) {
            .cargohub-checkout-info-table {
                display: block;
                overflow-x: auto;
            }
        }
        </style>
        <?php
    }

    /**
     * Check if COD payment method is selected.
     *
     * @since    1.0.0
     * @return   bool    Whether COD is selected.
     */
    private function is_cod_selected() {
        // Check if COD is the chosen payment method
        if (WC()->session && WC()->session->get('chosen_payment_method') === 'cod') {
            return true;
        }
        
        // Also check cart items for COD option
        if (function_exists('WC') && isset(WC()->cart)) {
            foreach (WC()->cart->get_cart() as $cart_item) {
                if (isset($cart_item['cargohub_cod_option']) && $cart_item['cargohub_cod_option'] === 'yes') {
                    return true;
                }
            }
        }
        
        return false;
    }

    /**
     * Check if there are courier products in the cart.
     *
     * @since    1.0.0
     * @return   bool    Whether there are courier products in the cart.
     */
    private function has_courier_products_in_cart() {
        if (!function_exists('WC') || !isset(WC()->cart)) {
            return false;
        }
        
        $cart_items = WC()->cart->get_cart();
        
        foreach ($cart_items as $cart_item) {
            if (isset($cart_item['cargohub_weight']) && isset($cart_item['cargohub_service_type'])) {
                return true;
            }
        }
        
        return false;
    }

    /**
     * Get the service type name based on service code.
     *
     * @since    1.0.0
     * @param    string    $service_code    The service type code.
     * @return   string                     The service type name.
     */
    private function get_service_type_name($service_code) {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $service_name = $wpdb->get_var($wpdb->prepare(
            "SELECT service_name FROM $table_name WHERE service_code = %s",
            $service_code
        ));
        
        if ($service_name) {
            return $service_name;
        }
        
        return $service_code;
    }
}