<?php
/**
 * The email-specific functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The email-specific functionality of the plugin.
 */
class CargoHub_Email {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Add dimension information to WooCommerce emails.
     *
     * @since    1.0.0
     * @param    WC_Order  $order           The order object.
     * @param    bool      $sent_to_admin   Whether the email is being sent to admin.
     * @param    bool      $plain_text      Whether the email is plain text.
     * @param    WC_Email  $email           The email object.
     */
    public function add_dimension_info_to_emails($order, $sent_to_admin, $plain_text, $email) {
        // Only proceed for order-related emails
        if (!in_array($email->id, array('customer_completed_order', 'customer_processing_order', 'new_order', 'customer_invoice'))) {
            return;
        }
        
        // Get order items
        $items = $order->get_items();
        $has_courier_items = false;
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        $dimension_unit = isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
        
        foreach ($items as $item) {
            $weight = $item->get_meta('cargohub_weight');
            $length = $item->get_meta('cargohub_length');
            $width = $item->get_meta('cargohub_width');
            $height = $item->get_meta('cargohub_height');
            
            if ($weight && $length && $width && $height) {
                $has_courier_items = true;
                break;
            }
        }
        
        if (!$has_courier_items) {
            return;
        }
        
        if ($plain_text) {
            $this->add_dimension_info_to_plain_text_email($order, $items, $weight_unit, $dimension_unit);
        } else {
            $this->add_dimension_info_to_html_email($order, $items, $weight_unit, $dimension_unit);
        }
    }

    /**
     * Add dimension information to plain text emails.
     *
     * @since    1.0.0
     * @param    WC_Order  $order           The order object.
     * @param    array     $items           The order items.
     * @param    string    $weight_unit     The weight unit.
     * @param    string    $dimension_unit  The dimension unit.
     */
    private function add_dimension_info_to_plain_text_email($order, $items, $weight_unit, $dimension_unit) {
        echo "\n\n" . __('Shipping Details:', 'cargohub') . "\n";
        echo "==========================================\n\n";
        
        // Display receiver phone if available
        $shipping_phone = get_post_meta($order->get_id(), '_shipping_phone', true);
        if ($shipping_phone) {
            echo __('Receiver Phone:', 'cargohub') . ' ' . $shipping_phone . "\n";
        }
        
        // Display shipping notes if available
        $shipping_notes = get_post_meta($order->get_id(), '_shipping_notes', true);
        if ($shipping_notes) {
            echo __('Delivery Instructions:', 'cargohub') . ' ' . $shipping_notes . "\n\n";
        }
        
        // Item details
        foreach ($items as $item) {
            $weight = $item->get_meta('cargohub_weight');
            $length = $item->get_meta('cargohub_length');
            $width = $item->get_meta('cargohub_width');
            $height = $item->get_meta('cargohub_height');
            $service_type = $item->get_meta('cargohub_service_type');
            $volumetric_weight = $item->get_meta('cargohub_volumetric_weight');
            
            if (!$weight || !$length || !$width || !$height) {
                continue; // Skip non-courier items
            }
            
            echo $item->get_name() . "\n";
            echo '  ' . __('Service:', 'cargohub') . ' ' . $this->get_service_type_name($service_type) . "\n";
            echo '  ' . __('Weight:', 'cargohub') . ' ' . $weight . ' ' . $weight_unit;
            
            if ($volumetric_weight && $volumetric_weight > $weight) {
                echo ' (' . __('Volumetric:', 'cargohub') . ' ' . $volumetric_weight . ' ' . $weight_unit . ')';
            }
            
            echo "\n";
            echo '  ' . __('Dimensions:', 'cargohub') . ' ' . $length . ' × ' . $width . ' × ' . $height . ' ' . $dimension_unit . "\n\n";
        }
    }

    /**
     * Add dimension information to HTML emails.
     *
     * @since    1.0.0
     * @param    WC_Order  $order           The order object.
     * @param    array     $items           The order items.
     * @param    string    $weight_unit     The weight unit.
     * @param    string    $dimension_unit  The dimension unit.
     */
    private function add_dimension_info_to_html_email($order, $items, $weight_unit, $dimension_unit) {
        echo '<h2>' . __('Shipping Details', 'cargohub') . '</h2>';
        
        // Display receiver phone if available
        $shipping_phone = get_post_meta($order->get_id(), '_shipping_phone', true);
        if ($shipping_phone) {
            echo '<p><strong>' . __('Receiver Phone:', 'cargohub') . '</strong> ' . esc_html($shipping_phone) . '</p>';
        }
        
        // Display shipping notes if available
        $shipping_notes = get_post_meta($order->get_id(), '_shipping_notes', true);
        if ($shipping_notes) {
            echo '<p><strong>' . __('Delivery Instructions:', 'cargohub') . '</strong> ' . nl2br(esc_html($shipping_notes)) . '</p>';
        }
        
        echo '<table class="td" cellspacing="0" cellpadding="6" style="width: 100%; font-family: \'Helvetica Neue\', Helvetica, Roboto, Arial, sans-serif; margin-bottom: 40px;" border="1">';
        
        echo '<thead>';
        echo '<tr>';
        echo '<th class="td" scope="col" style="text-align:left;">' . __('Product', 'cargohub') . '</th>';
        echo '<th class="td" scope="col" style="text-align:left;">' . __('Service', 'cargohub') . '</th>';
        echo '<th class="td" scope="col" style="text-align:left;">' . __('Weight', 'cargohub') . '</th>';
        echo '<th class="td" scope="col" style="text-align:left;">' . __('Dimensions', 'cargohub') . '</th>';
        echo '</tr>';
        echo '</thead>';
        
        echo '<tbody>';
        
        foreach ($items as $item) {
            $weight = $item->get_meta('cargohub_weight');
            $length = $item->get_meta('cargohub_length');
            $width = $item->get_meta('cargohub_width');
            $height = $item->get_meta('cargohub_height');
            $service_type = $item->get_meta('cargohub_service_type');
            $volumetric_weight = $item->get_meta('cargohub_volumetric_weight');
            
            if (!$weight || !$length || !$width || !$height) {
                continue; // Skip non-courier items
            }
            
            echo '<tr>';
            
            // Product name
            echo '<td class="td" style="text-align:left; vertical-align:middle; border: 1px solid #eee; word-wrap:break-word;">' . esc_html($item->get_name()) . '</td>';
            
            // Service type
            echo '<td class="td" style="text-align:left; vertical-align:middle; border: 1px solid #eee;">' . $this->get_service_type_name($service_type) . '</td>';
            
            // Weight
            echo '<td class="td" style="text-align:left; vertical-align:middle; border: 1px solid #eee;">';
            echo esc_html($weight) . ' ' . esc_html($weight_unit);
            if ($volumetric_weight && $volumetric_weight > $weight) {
                echo '<br><small>' . __('Volumetric:', 'cargohub') . ' ' . esc_html($volumetric_weight) . ' ' . esc_html($weight_unit) . '</small>';
            }
            echo '</td>';
            
            // Dimensions
            echo '<td class="td" style="text-align:left; vertical-align:middle; border: 1px solid #eee;">' . esc_html($length) . ' × ' . esc_html($width) . ' × ' . esc_html($height) . ' ' . esc_html($dimension_unit) . '</td>';
            
            echo '</tr>';
        }
        
        echo '</tbody>';
        echo '</table>';
    }

    /**
     * Add bank details to order emails.
     *
     * @since    1.0.0
     * @param    WC_Order $order The order object.
     * @param    bool $sent_to_admin Whether the email is sent to admin.
     * @param    bool $plain_text Whether the email is plain text.
     * @param    WC_Email $email The email object.
     */
    public function add_bank_details_to_emails($order, $sent_to_admin, $plain_text, $email) {
        // Only add to specific emails
        if (!in_array($email->id, array('customer_completed_order', 'customer_processing_order', 'customer_on_hold_order'))) {
            return;
        }
        
        // Check if this is a COD order
        if ($order->get_payment_method() !== 'cod') {
            return;
        }
        
        // Get bank details
        $account_name = get_post_meta($order->get_id(), '_bank_account_name', true);
        $account_number = get_post_meta($order->get_id(), '_bank_account_number', true);
        $ifsc_code = get_post_meta($order->get_id(), '_bank_ifsc_code', true);
        
        // Only proceed if we have bank details
        if (empty($account_name) && empty($account_number) && empty($ifsc_code)) {
            return;
        }
        
        // Display the bank details
        if ($plain_text) {
            echo "\n\n==========\n\n";
            echo __('Bank Account Details', 'cargohub') . "\n\n";
            
            if (!empty($account_name)) {
                echo __('Account Holder Name', 'cargohub') . ': ' . $account_name . "\n";
            }
            
            if (!empty($account_number)) {
                echo __('Account Number', 'cargohub') . ': ' . $account_number . "\n";
            }
            
            if (!empty($ifsc_code)) {
                echo __('IFSC Code', 'cargohub') . ': ' . $ifsc_code . "\n";
            }
        } else {
            echo '<h2>' . __('Bank Account Details', 'cargohub') . '</h2>';
            echo '<table class="td" cellspacing="0" cellpadding="6" style="width: 100%; margin-bottom: 20px;">';
            
            // Account Name
            if (!empty($account_name)) {
                echo '<tr>';
                echo '<th style="text-align: left; border-bottom: 1px solid #ececec;">' . __('Account Holder Name', 'cargohub') . ':</th>';
                echo '<td style="text-align: left; border-bottom: 1px solid #ececec;">' . esc_html($account_name) . '</td>';
                echo '</tr>';
            }
            
            // Account Number
            if (!empty($account_number)) {
                echo '<tr>';
                echo '<th style="text-align: left; border-bottom: 1px solid #ececec;">' . __('Account Number', 'cargohub') . ':</th>';
                echo '<td style="text-align: left; border-bottom: 1px solid #ececec;">' . esc_html($account_number) . '</td>';
                echo '</tr>';
            }
            
            // IFSC Code
            if (!empty($ifsc_code)) {
                echo '<tr>';
                echo '<th style="text-align: left; border-bottom: 1px solid #ececec;">' . __('IFSC Code', 'cargohub') . ':</th>';
                echo '<td style="text-align: left; border-bottom: 1px solid #ececec;">' . esc_html($ifsc_code) . '</td>';
                echo '</tr>';
            }
            
            echo '</table>';
        }
    }

    /**
     * Get the service type name based on service code.
     *
     * @since    1.0.0
     * @param    string    $service_code    The service type code.
     * @return   string                     The service type name.
     */
    private function get_service_type_name($service_code) {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $service_name = $wpdb->get_var($wpdb->prepare(
            "SELECT service_name FROM $table_name WHERE service_code = %s",
            $service_code
        ));
        
        if ($service_name) {
            return $service_name;
        }
        
        // Fallback to common service types if not found in database
        $service_types = array(
            'standard' => __('Standard Delivery', 'cargohub'),
            'express' => __('Express Delivery', 'cargohub'),
            'priority' => __('Priority Delivery', 'cargohub'),
            'economy' => __('Economy Delivery', 'cargohub'),
        );
        
        if (isset($service_types[$service_code])) {
            return $service_types[$service_code];
        }
        
        return $service_code;
    }
}