<?php
/**
 * The order-specific functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The order-specific functionality of the plugin.
 */
class CargoHub_Order {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Process a courier order after checkout.
     *
     * @since    1.0.0
     * @param    int       $order_id    The order ID.
     * @param    array     $posted_data The posted data.
     * @param    WC_Order  $order       The order object.
     */
    public function process_courier_order($order_id, $posted_data, $order) {
        // Get order items
        $items = $order->get_items();
        
        foreach ($items as $item) {
            // Get the cart item data
            $cart_item_data = $item->get_meta('_cargohub_data');
            
            if ($cart_item_data) {
                // Store the data as order item meta
                if (isset($cart_item_data['cargohub_weight'])) {
                    $item->update_meta_data('cargohub_weight', $cart_item_data['cargohub_weight']);
                }
                
                if (isset($cart_item_data['cargohub_length'])) {
                    $item->update_meta_data('cargohub_length', $cart_item_data['cargohub_length']);
                }
                
                if (isset($cart_item_data['cargohub_width'])) {
                    $item->update_meta_data('cargohub_width', $cart_item_data['cargohub_width']);
                }
                
                if (isset($cart_item_data['cargohub_height'])) {
                    $item->update_meta_data('cargohub_height', $cart_item_data['cargohub_height']);
                }
                
                if (isset($cart_item_data['cargohub_service_type'])) {
                    $item->update_meta_data('cargohub_service_type', $cart_item_data['cargohub_service_type']);
                }
                
                if (isset($cart_item_data['cargohub_cod_option'])) {
                    $item->update_meta_data('cargohub_cod_option', $cart_item_data['cargohub_cod_option']);
                }
                
                if (isset($cart_item_data['cargohub_volumetric_weight'])) {
                    $item->update_meta_data('cargohub_volumetric_weight', $cart_item_data['cargohub_volumetric_weight']);
                }
                
                if (isset($cart_item_data['cargohub_chargeable_weight'])) {
                    $item->update_meta_data('cargohub_chargeable_weight', $cart_item_data['cargohub_chargeable_weight']);
                }
                
                if (isset($cart_item_data['cargohub_shipping_cost'])) {
                    $item->update_meta_data('cargohub_shipping_cost', $cart_item_data['cargohub_shipping_cost']);
                }
                
                if (isset($cart_item_data['cargohub_cod_fee'])) {
                    $item->update_meta_data('cargohub_cod_fee', $cart_item_data['cargohub_cod_fee']);
                }
                
                if (isset($cart_item_data['cargohub_total_cost'])) {
                    $item->update_meta_data('cargohub_total_cost', $cart_item_data['cargohub_total_cost']);
                }
                
                $item->save();
            }
        }
        
        // Store order meta for easier querying
        update_post_meta($order_id, '_is_cargohub_order', 'yes');
        
        // Optionally store the receiver phone number as order meta
        $shipping_phone = get_post_meta($order_id, '_shipping_phone', true);
        if ($shipping_phone) {
            update_post_meta($order_id, '_cargohub_receiver_phone', $shipping_phone);
        }
        
        // Do other post-order processing here
        do_action('cargohub_after_order_processed', $order_id, $order);
    }

    /**
     * Display dimension information in the order admin page.
     *
     * @since    1.0.0
     * @param    WC_Order  $order  The order object.
     */
    public function display_dimension_info_in_order($order) {
        // Get order items
        $items = $order->get_items();
        $has_courier_items = false;
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        $dimension_unit = isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
        
        foreach ($items as $item) {
            $weight = $item->get_meta('cargohub_weight');
            $length = $item->get_meta('cargohub_length');
            $width = $item->get_meta('cargohub_width');
            $height = $item->get_meta('cargohub_height');
            $service_type = $item->get_meta('cargohub_service_type');
            $cod_option = $item->get_meta('cargohub_cod_option');
            
            if ($weight && $length && $width && $height) {
                $has_courier_items = true;
                break;
            }
        }
        
        if (!$has_courier_items) {
            return;
        }
        
        // Add a heading for courier information
        echo '<h3>' . __('Courier Information', 'cargohub') . '</h3>';
        
        // Display receiver phone if available
        $shipping_phone = get_post_meta($order->get_id(), '_shipping_phone', true);
        if ($shipping_phone) {
            echo '<p><strong>' . __('Receiver Phone:', 'cargohub') . '</strong> ' . esc_html($shipping_phone) . '</p>';
        }
        
        // Display shipping notes if available
        $shipping_notes = get_post_meta($order->get_id(), '_shipping_notes', true);
        if ($shipping_notes) {
            echo '<p><strong>' . __('Delivery Instructions:', 'cargohub') . '</strong> ' . nl2br(esc_html($shipping_notes)) . '</p>';
        }
        
        // Start the table
        echo '<table class="widefat fixed" style="width:100%; margin-bottom:20px;">';
        
        // Table header
        echo '<thead>';
        echo '<tr>';
        echo '<th>' . __('Product', 'cargohub') . '</th>';
        echo '<th>' . __('Service', 'cargohub') . '</th>';
        echo '<th>' . __('Weight', 'cargohub') . '</th>';
        echo '<th>' . __('Dimensions', 'cargohub') . '</th>';
        echo '<th>' . __('COD', 'cargohub') . '</th>';
        echo '</tr>';
        echo '</thead>';
        
        // Table body
        echo '<tbody>';
        
        foreach ($items as $item) {
            $weight = $item->get_meta('cargohub_weight');
            $length = $item->get_meta('cargohub_length');
            $width = $item->get_meta('cargohub_width');
            $height = $item->get_meta('cargohub_height');
            $service_type = $item->get_meta('cargohub_service_type');
            $cod_option = $item->get_meta('cargohub_cod_option');
            $volumetric_weight = $item->get_meta('cargohub_volumetric_weight');
            
            if (!$weight || !$length || !$width || !$height) {
                continue; // Skip non-courier items
            }
            
            echo '<tr>';
            
            // Product name
            echo '<td>' . esc_html($item->get_name()) . '</td>';
            
            // Service type
            echo '<td>' . $this->get_service_type_name($service_type) . '</td>';
            
            // Weight
            echo '<td>';
            echo esc_html($weight) . ' ' . esc_html($weight_unit);
            if ($volumetric_weight && $volumetric_weight > $weight) {
                echo '<br><small>' . __('Volumetric:', 'cargohub') . ' ' . esc_html($volumetric_weight) . ' ' . esc_html($weight_unit) . '</small>';
            }
            echo '</td>';
            
            // Dimensions
            echo '<td>' . esc_html($length) . ' × ' . esc_html($width) . ' × ' . esc_html($height) . ' ' . esc_html($dimension_unit) . '</td>';
            
            // COD option
            echo '<td>' . ($cod_option === 'yes' ? __('Yes', 'cargohub') : __('No', 'cargohub')) . '</td>';
            
            echo '</tr>';
        }
        
        echo '</tbody>';
        echo '</table>';
    }

    /**
     * Display bank details in the order details page.
     *
     * @since    1.0.0
     * @param    WC_Order $order The order object.
     */
    public function display_bank_details_in_order($order) {
        // Check if this is a COD order
        if ($order->get_payment_method() !== 'cod') {
            return;
        }
        
        // Get bank details
        $account_name = get_post_meta($order->get_id(), '_bank_account_name', true);
        $account_number = get_post_meta($order->get_id(), '_bank_account_number', true);
        $ifsc_code = get_post_meta($order->get_id(), '_bank_ifsc_code', true);
        
        // Only proceed if we have bank details
        if (empty($account_name) && empty($account_number) && empty($ifsc_code)) {
            return;
        }
        
        // Display the bank details
        echo '<h3>' . __('Bank Account Details', 'cargohub') . '</h3>';
        echo '<table class="woocommerce-table shop_table bank_details">';
        
        // Account Name
        if (!empty($account_name)) {
            echo '<tr>';
            echo '<th>' . __('Account Holder Name', 'cargohub') . ':</th>';
            echo '<td>' . esc_html($account_name) . '</td>';
            echo '</tr>';
        }
        
        // Account Number
        if (!empty($account_number)) {
            echo '<tr>';
            echo '<th>' . __('Account Number', 'cargohub') . ':</th>';
            echo '<td>' . esc_html($account_number) . '</td>';
            echo '</tr>';
        }
        
        // IFSC Code
        if (!empty($ifsc_code)) {
            echo '<tr>';
            echo '<th>' . __('IFSC Code', 'cargohub') . ':</th>';
            echo '<td>' . esc_html($ifsc_code) . '</td>';
            echo '</tr>';
        }
        
        echo '</table>';
    }

    /**
     * Display dimension information in the order details page on the front-end.
     *
     * @since    1.0.0
     * @param    WC_Order  $order  The order object.
     */
    public function display_dimension_info_in_order_details($order) {
        // Get order items
        $items = $order->get_items();
        $has_courier_items = false;
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        $dimension_unit = isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
        
        foreach ($items as $item) {
            $weight = $item->get_meta('cargohub_weight');
            $length = $item->get_meta('cargohub_length');
            $width = $item->get_meta('cargohub_width');
            $height = $item->get_meta('cargohub_height');
            
            if ($weight && $length && $width && $height) {
                $has_courier_items = true;
                break;
            }
        }
        
        if (!$has_courier_items) {
            return;
        }
        
        // Add a heading
        echo '<h2>' . __('Shipping Details', 'cargohub') . '</h2>';
        
        // Start the table
        echo '<table class="woocommerce-table woocommerce-table--courier-details shop_table courier_details">';
        
        // Table header
        echo '<thead>';
        echo '<tr>';
        echo '<th>' . __('Product', 'cargohub') . '</th>';
        echo '<th>' . __('Service', 'cargohub') . '</th>';
        echo '<th>' . __('Weight', 'cargohub') . '</th>';
        echo '<th>' . __('Dimensions', 'cargohub') . '</th>';
        echo '</tr>';
        echo '</thead>';
        
        // Table body
        echo '<tbody>';
        
        foreach ($items as $item) {
            $weight = $item->get_meta('cargohub_weight');
            $length = $item->get_meta('cargohub_length');
            $width = $item->get_meta('cargohub_width');
            $height = $item->get_meta('cargohub_height');
            $service_type = $item->get_meta('cargohub_service_type');
            $volumetric_weight = $item->get_meta('cargohub_volumetric_weight');
            
            if (!$weight || !$length || !$width || !$height) {
                continue; // Skip non-courier items
            }
            
            echo '<tr>';
            
            // Product name
            echo '<td>' . esc_html($item->get_name()) . '</td>';
            
            // Service type
            echo '<td>' . $this->get_service_type_name($service_type) . '</td>';
            
            // Weight
            echo '<td>';
            echo esc_html($weight) . ' ' . esc_html($weight_unit);
            if ($volumetric_weight && $volumetric_weight > $weight) {
                echo '<br><small>' . __('Volumetric:', 'cargohub') . ' ' . esc_html($volumetric_weight) . ' ' . esc_html($weight_unit) . '</small>';
            }
            echo '</td>';
            
            // Dimensions
            echo '<td>' . esc_html($length) . ' × ' . esc_html($width) . ' × ' . esc_html($height) . ' ' . esc_html($dimension_unit) . '</td>';
            
            echo '</tr>';
        }
        
        echo '</tbody>';
        echo '</table>';
    }

    /**
     * Get the service type name based on service code.
     *
     * @since    1.0.0
     * @param    string    $service_code    The service type code.
     * @return   string                     The service type name.
     */
    private function get_service_type_name($service_code) {
        global $wpdb;
        
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $service_name = $wpdb->get_var($wpdb->prepare(
            "SELECT service_name FROM $table_name WHERE service_code = %s",
            $service_code
        ));
        
        if ($service_name) {
            return $service_name;
        }
        
        // Fallback to common service types if not found in database
        $service_types = array(
            'standard' => __('Standard Delivery', 'cargohub'),
            'express' => __('Express Delivery', 'cargohub'),
            'priority' => __('Priority Delivery', 'cargohub'),
            'economy' => __('Economy Delivery', 'cargohub'),
        );
        
        if (isset($service_types[$service_code])) {
            return $service_types[$service_code];
        }
        
        return $service_code;
    }
}