<?php
/**
 * The product-specific functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The product-specific functionality of the plugin.
 *
 * Handles all custom fields and calculations on the product page.
 */
class CargoHub_Product {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param    string    $plugin_name       The name of this plugin.
     * @param    string    $version           The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Hide the default price and show the lowest rate as a starting price.
     *
     * @since    1.0.0
     */
    public function customize_price_display() {
        global $post;
        
        // Check if this is a courier product
        if (!$this->is_courier_product($post->ID)) {
            return;
        }
        
        // Remove the default price display
        remove_action('woocommerce_single_product_summary', 'woocommerce_template_single_price', 10);
        
        // Add our custom price display
        add_action('woocommerce_single_product_summary', array($this, 'display_lowest_rate'), 6);
    }

    /**
     * Display the lowest shipping rate as a starting price.
     *
     * @since    1.0.0
     */
    public function display_lowest_rate() {
        global $wpdb;
        
        // Get the lowest rate from the database
        $rates_table = $wpdb->prefix . 'cargohub_rates';
        $lowest_rate = $wpdb->get_var("SELECT MIN(price) FROM $rates_table");
        
        // If no rates found, return
        if (!$lowest_rate) {
            return;
        }
        
        // Output the custom price display
        echo '<div class="cargohub-starting-price">';
        echo '<p class="price starting-from">' . esc_html__('Starting from', 'cargohub') . ' <span class="amount">' . wc_price($lowest_rate) . '</span></p>';
        echo '<p class="cargohub-price-tagline">' . esc_html__('Final price calculated based on weight, dimensions & service type', 'cargohub') . '</p>';
        echo '</div>';
    }

    /**
     * Remove quantity field from product page for courier products.
     *
     * @since    1.0.0
     */
    public function remove_quantity_field() {
        global $post;
        
        // Check if this is a courier product
        if ($this->is_courier_product($post->ID)) {
            // Remove the quantity field
            add_filter('woocommerce_is_sold_individually', function($sold_individually, $product) use ($post) {
                if ($product->get_id() == $post->ID) {
                    return true; // Forces quantity to 1 and hides the input
                }
                return $sold_individually;
            }, 10, 2);
        }
    }

    /**
     * Add dimension fields to the product page.
     *
     * @since    1.0.0
     */
    public function add_dimension_fields() {
        global $post;
        
        // Check if this is a courier product
        if (!$this->is_courier_product($post->ID)) {
            return;
        }
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $weight_unit = isset($options['weight_unit']) ? $options['weight_unit'] : 'kg';
        $dimension_unit = isset($options['dimension_unit']) ? $options['dimension_unit'] : 'cm';
        $min_weight = isset($options['min_weight']) ? $options['min_weight'] : 0.5;
        $max_weight = isset($options['max_weight']) ? $options['max_weight'] : 30;
        
        // Get saved values if editing a product in the cart
        $weight = isset($_REQUEST['cargohub_weight']) ? wc_clean(wp_unslash($_REQUEST['cargohub_weight'])) : '';
        $length = isset($_REQUEST['cargohub_length']) ? wc_clean(wp_unslash($_REQUEST['cargohub_length'])) : '';
        $width = isset($_REQUEST['cargohub_width']) ? wc_clean(wp_unslash($_REQUEST['cargohub_width'])) : '';
        $height = isset($_REQUEST['cargohub_height']) ? wc_clean(wp_unslash($_REQUEST['cargohub_height'])) : '';
        
        // Output dimension fields
        wc_get_template(
            'product/dimension-fields.php', 
            array(
                'weight_unit' => $weight_unit,
                'dimension_unit' => $dimension_unit,
                'min_weight' => $min_weight,
                'max_weight' => $max_weight,
                'weight' => $weight,
                'length' => $length,
                'width' => $width,
                'height' => $height
            ),
            'cargohub/',
            CARGOHUB_PLUGIN_DIR . 'templates/'
        );
    }
    
    /**
     * Add service type selector to the product page.
     *
     * @since    1.0.0
     */
    public function add_service_type_selector() {
        global $post;
        
        // Check if this is a courier product
        if (!$this->is_courier_product($post->ID)) {
            return;
        }
        
        // Get service types
        global $wpdb;
        $table_name = $wpdb->prefix . 'cargohub_service_types';
        $service_types = $wpdb->get_results("SELECT * FROM $table_name WHERE active = 1 ORDER BY service_name ASC");
        
        if (empty($service_types)) {
            return;
        }
        
        // Get saved value if editing a product in the cart
        $selected_service = isset($_REQUEST['cargohub_service_type']) ? wc_clean(wp_unslash($_REQUEST['cargohub_service_type'])) : '';
        
        // Output service type selector
        wc_get_template(
            'product/service-type-selector.php', 
            array(
                'service_types' => $service_types,
                'selected_service' => $selected_service
            ),
            'cargohub/',
            CARGOHUB_PLUGIN_DIR . 'templates/'
        );
    }
    
    /**
     * Add COD option to the product page.
     *
     * @since    1.0.0
     */
    public function add_cod_option() {
        global $post;
        
        // Check if this is a courier product
        if (!$this->is_courier_product($post->ID)) {
            return;
        }
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $cod_fee_percentage = isset($options['cod_fee_percentage']) ? $options['cod_fee_percentage'] : 10;
        
        // Get saved value if editing a product in the cart
        $cod_selected = isset($_REQUEST['cargohub_cod_option']) ? wc_clean(wp_unslash($_REQUEST['cargohub_cod_option'])) === 'yes' : false;
        
        // Output COD option
        wc_get_template(
            'product/cod-option.php', 
            array(
                'cod_fee_percentage' => $cod_fee_percentage,
                'cod_selected' => $cod_selected
            ),
            'cargohub/',
            CARGOHUB_PLUGIN_DIR . 'templates/'
        );
    }
    
    /**
     * Validate dimensions when adding to cart.
     *
     * @since    1.0.0
     * @param    bool     $passed            Whether validation has passed.
     * @param    int      $product_id        The product ID.
     * @param    int      $quantity          The quantity.
     * @return   bool                        Whether validation has passed.
     */
    public function validate_dimensions($passed, $product_id, $quantity) {
        // Check if this is a courier product
        if (!$this->is_courier_product($product_id)) {
            return $passed;
        }
        
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $min_weight = isset($options['min_weight']) ? $options['min_weight'] : 0.5;
        $max_weight = isset($options['max_weight']) ? $options['max_weight'] : 30;
        
        // Validate weight
        if (!isset($_POST['cargohub_weight']) || empty($_POST['cargohub_weight'])) {
            wc_add_notice(__('Please enter the package weight.', 'cargohub'), 'error');
            return false;
        }
        
        $weight = wc_format_decimal($_POST['cargohub_weight']);
        
        if ($weight < $min_weight) {
            // Translators: %s is the minimum weight
            wc_add_notice(sprintf(__('The minimum allowed weight is %s.', 'cargohub'), $min_weight), 'error');
            return false;
        }
        
        if ($weight > $max_weight) {
            // Translators: %s is the maximum weight
            wc_add_notice(sprintf(__('The maximum allowed weight is %s.', 'cargohub'), $max_weight), 'error');
            return false;
        }
        
        // Validate dimensions
        if (!isset($_POST['cargohub_length']) || empty($_POST['cargohub_length'])) {
            wc_add_notice(__('Please enter the package length.', 'cargohub'), 'error');
            return false;
        }
        
        if (!isset($_POST['cargohub_width']) || empty($_POST['cargohub_width'])) {
            wc_add_notice(__('Please enter the package width.', 'cargohub'), 'error');
            return false;
        }
        
        if (!isset($_POST['cargohub_height']) || empty($_POST['cargohub_height'])) {
            wc_add_notice(__('Please enter the package height.', 'cargohub'), 'error');
            return false;
        }
        
        // Validate service type
        if (!isset($_POST['cargohub_service_type']) || empty($_POST['cargohub_service_type'])) {
            wc_add_notice(__('Please select a delivery service type.', 'cargohub'), 'error');
            return false;
        }
        
        return $passed;
    }
    
    /**
     * Add custom fields to cart item data.
     *
     * @since    1.0.0
     * @param    array    $cart_item_data    The cart item data.
     * @param    int      $product_id        The product ID.
     * @param    int      $variation_id      The variation ID.
     * @return   array                       The modified cart item data.
     */
    public function add_custom_fields_to_cart_item($cart_item_data, $product_id, $variation_id) {
        // Check if this is a courier product
        if (!$this->is_courier_product($product_id)) {
            return $cart_item_data;
        }
        
        // Get dimensions
        if (isset($_POST['cargohub_weight']) && !empty($_POST['cargohub_weight'])) {
            $cart_item_data['cargohub_weight'] = wc_format_decimal($_POST['cargohub_weight']);
        }
        
        if (isset($_POST['cargohub_length']) && !empty($_POST['cargohub_length'])) {
            $cart_item_data['cargohub_length'] = wc_format_decimal($_POST['cargohub_length']);
        }
        
        if (isset($_POST['cargohub_width']) && !empty($_POST['cargohub_width'])) {
            $cart_item_data['cargohub_width'] = wc_format_decimal($_POST['cargohub_width']);
        }
        
        if (isset($_POST['cargohub_height']) && !empty($_POST['cargohub_height'])) {
            $cart_item_data['cargohub_height'] = wc_format_decimal($_POST['cargohub_height']);
        }
        
        // Get service type
        if (isset($_POST['cargohub_service_type']) && !empty($_POST['cargohub_service_type'])) {
            $cart_item_data['cargohub_service_type'] = sanitize_text_field($_POST['cargohub_service_type']);
        }
        
        // Get COD option
        $cart_item_data['cargohub_cod_option'] = isset($_POST['cargohub_cod_option']) ? 'yes' : 'no';
        
        // Calculate volumetric weight
        if (isset($cart_item_data['cargohub_length']) && isset($cart_item_data['cargohub_width']) && isset($cart_item_data['cargohub_height'])) {
            $options = get_option('cargohub_general_settings', array());
            $enable_volumetric = isset($options['enable_volumetric_weight']) ? $options['enable_volumetric_weight'] : 'yes';
            $volumetric_divisor = isset($options['volumetric_divisor']) ? $options['volumetric_divisor'] : 5000;
            
            if ($enable_volumetric === 'yes') {
                $volumetric_weight = ($cart_item_data['cargohub_length'] * $cart_item_data['cargohub_width'] * $cart_item_data['cargohub_height']) / $volumetric_divisor;
                $cart_item_data['cargohub_volumetric_weight'] = wc_format_decimal($volumetric_weight, 2);
                
                // Use the greater of actual weight or volumetric weight
                $cart_item_data['cargohub_chargeable_weight'] = max($cart_item_data['cargohub_weight'], $cart_item_data['cargohub_volumetric_weight']);
            } else {
                $cart_item_data['cargohub_chargeable_weight'] = $cart_item_data['cargohub_weight'];
            }
        }
        
        // Calculate shipping cost
        $cart_item_data['cargohub_shipping_cost'] = $this->calculate_shipping_cost(
            $cart_item_data['cargohub_chargeable_weight'],
            $cart_item_data['cargohub_service_type']
        );
        
        // Apply COD fee if selected
        if ($cart_item_data['cargohub_cod_option'] === 'yes') {
            $options = get_option('cargohub_general_settings', array());
            $cod_fee_percentage = isset($options['cod_fee_percentage']) ? $options['cod_fee_percentage'] : 10;
            
            $cod_fee = ($cart_item_data['cargohub_shipping_cost'] * $cod_fee_percentage) / 100;
            $cart_item_data['cargohub_cod_fee'] = wc_format_decimal($cod_fee, 2);
            $cart_item_data['cargohub_total_cost'] = $cart_item_data['cargohub_shipping_cost'] + $cart_item_data['cargohub_cod_fee'];
        } else {
            $cart_item_data['cargohub_cod_fee'] = 0;
            $cart_item_data['cargohub_total_cost'] = $cart_item_data['cargohub_shipping_cost'];
        }
        
        // Make sure a new cart item is added even if custom data is the same
        $cart_item_data['unique_key'] = md5(microtime() . rand());
        
        return $cart_item_data;
    }
    
    /**
     * Calculate the shipping cost based on weight and service type.
     *
     * @since    1.0.0
     * @param    float    $weight         The weight of the package.
     * @param    string   $service_type   The service type code.
     * @return   float                    The shipping cost.
     */
    private function calculate_shipping_cost($weight, $service_type) {
        global $wpdb;
        
        // Get customer's country
        $customer_country = WC()->customer->get_shipping_country();
        if (empty($customer_country)) {
            $customer_country = WC()->customer->get_billing_country();
        }
        
        // If still empty, use store base country
        if (empty($customer_country)) {
            $customer_country = WC()->countries->get_base_country();
        }
        
        // Find the zone for the customer's country
        $zones_table = $wpdb->prefix . 'cargohub_zones';
        $zone_rows = $wpdb->get_results("SELECT id, countries FROM $zones_table");
        
        $zone_id = 0;
        foreach ($zone_rows as $zone) {
            $countries = unserialize($zone->countries);
            if (in_array($customer_country, $countries)) {
                $zone_id = $zone->id;
                break;
            }
        }
        
        // If no zone found, use the first zone as default
        if ($zone_id === 0 && !empty($zone_rows)) {
            $zone_id = $zone_rows[0]->id;
        }
        
        // Get rate for this zone, service type, and weight
        $rates_table = $wpdb->prefix . 'cargohub_rates';
        $rate = $wpdb->get_row($wpdb->prepare(
            "SELECT price FROM $rates_table 
            WHERE zone_id = %d 
            AND service_type = %s 
            AND weight_from <= %f 
            AND weight_to > %f 
            ORDER BY price ASC 
            LIMIT 1",
            $zone_id,
            $service_type,
            $weight,
            $weight
        ));
        
        if ($rate) {
            return $rate->price;
        }
        
        // If no specific rate found, find the highest weight range
        $highest_rate = $wpdb->get_row($wpdb->prepare(
            "SELECT price FROM $rates_table 
            WHERE zone_id = %d 
            AND service_type = %s 
            ORDER BY weight_to DESC 
            LIMIT 1",
            $zone_id,
            $service_type
        ));
        
        if ($highest_rate) {
            return $highest_rate->price;
        }
        
        // If still no rate found, return a default price
        return 0;
    }
    
    /**
     * Check if a product is a courier product.
     *
     * @since    1.0.0
     * @param    int       $product_id    The product ID to check.
     * @return   bool                     Whether the product is a courier product.
     */
    private function is_courier_product($product_id) {
        // Get settings
        $options = get_option('cargohub_general_settings', array());
        $courier_products = isset($options['courier_products']) ? $options['courier_products'] : array();
        
        // Check if the product is in the list of courier products
        return in_array($product_id, $courier_products);
    }
}