<?php
/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The core plugin class.
 */
class CargoHub {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      CargoHub_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        $this->version = CARGOHUB_VERSION;
        $this->plugin_name = 'cargohub';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
        $this->define_woocommerce_hooks();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - CargoHub_Loader. Orchestrates the hooks of the plugin.
     * - CargoHub_i18n. Defines internationalization functionality.
     * - CargoHub_Admin. Defines all hooks for the admin area.
     * - CargoHub_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {
        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once CARGOHUB_PLUGIN_DIR . 'includes/class-cargohub-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once CARGOHUB_PLUGIN_DIR . 'includes/class-cargohub-i18n.php';

        /**
         * The classes responsible for defining all actions that occur in the admin area.
         */
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/class-cargohub-admin.php';
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/class-cargohub-settings.php';
        require_once CARGOHUB_PLUGIN_DIR . 'includes/admin/class-cargohub-admin-orders.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once CARGOHUB_PLUGIN_DIR . 'includes/public/class-cargohub-public.php';
        
        /**
         * The class responsible for defining all WooCommerce related functionality.
         */
        require_once CARGOHUB_PLUGIN_DIR . 'includes/class-cargohub-product.php';
        require_once CARGOHUB_PLUGIN_DIR . 'includes/class-cargohub-order.php';
        require_once CARGOHUB_PLUGIN_DIR . 'includes/class-cargohub-cart.php';
        require_once CARGOHUB_PLUGIN_DIR . 'includes/class-cargohub-checkout.php';
        require_once CARGOHUB_PLUGIN_DIR . 'includes/class-cargohub-email.php';

        $this->loader = new CargoHub_Loader();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the CargoHub_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {
        $plugin_i18n = new CargoHub_i18n();
        $plugin_i18n->set_domain($this->get_plugin_name());

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {
        $plugin_admin = new CargoHub_Admin($this->get_plugin_name(), $this->get_version());
        $plugin_settings = new CargoHub_Settings($this->get_plugin_name(), $this->get_version());
        $plugin_admin_orders = new CargoHub_Admin_Orders($this->get_plugin_name(), $this->get_version());

        // Admin assets
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

        // Admin menu
        $this->loader->add_action('admin_menu', $plugin_admin, 'add_admin_menu');
        
        // Settings
        $this->loader->add_action('admin_init', $plugin_settings, 'register_settings');
        
        // Admin AJAX handlers
        $this->loader->add_action('wp_ajax_cargohub_admin_action', $plugin_admin, 'handle_ajax');
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {
        $plugin_public = new CargoHub_Public($this->get_plugin_name(), $this->get_version());

        // Public assets
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
        
        // AJAX handlers for frontend
        $this->loader->add_action('wp_ajax_cargohub_calculate_price', $plugin_public, 'calculate_price');
        $this->loader->add_action('wp_ajax_nopriv_cargohub_calculate_price', $plugin_public, 'calculate_price');
    }

    /**
     * Register all of the hooks related to WooCommerce functionality.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_woocommerce_hooks() {
        $plugin_product = new CargoHub_Product($this->get_plugin_name(), $this->get_version());
        $plugin_cart = new CargoHub_Cart($this->get_plugin_name(), $this->get_version());
        $plugin_checkout = new CargoHub_Checkout($this->get_plugin_name(), $this->get_version());
        $plugin_order = new CargoHub_Order($this->get_plugin_name(), $this->get_version());
        $plugin_email = new CargoHub_Email($this->get_plugin_name(), $this->get_version());

        // Product page customization
        $this->loader->add_action('woocommerce_before_single_product', $plugin_product, 'remove_quantity_field');
        $this->loader->add_action('woocommerce_before_add_to_cart_button', $plugin_product, 'add_dimension_fields');
        $this->loader->add_action('woocommerce_before_add_to_cart_button', $plugin_product, 'add_service_type_selector');
        $this->loader->add_action('woocommerce_before_add_to_cart_button', $plugin_product, 'add_cod_option');
        $this->loader->add_action('woocommerce_before_single_product', $plugin_product, 'customize_price_display', 5);
        $this->loader->add_filter('woocommerce_add_to_cart_validation', $plugin_product, 'validate_dimensions', 10, 3);
        $this->loader->add_filter('woocommerce_add_cart_item_data', $plugin_product, 'add_custom_fields_to_cart_item', 10, 3);
        
        // Cart customization
        $this->loader->add_filter('woocommerce_cart_item_price', $plugin_cart, 'custom_cart_item_price', 10, 3);
        $this->loader->add_filter('woocommerce_cart_item_subtotal', $plugin_cart, 'custom_cart_item_subtotal', 10, 3);
        $this->loader->add_action('woocommerce_after_cart_item_name', $plugin_cart, 'add_dimension_info_to_cart', 10, 2);
        $this->loader->add_filter('woocommerce_cart_item_quantity', $plugin_cart, 'hide_quantity_input', 10, 3);
        $this->loader->add_filter('woocommerce_update_cart_action_cart_updated', $plugin_cart, 'prevent_quantity_update', 10, 1);
        
        // Price adjustment hooks
        $this->loader->add_filter('woocommerce_product_get_price', $plugin_cart, 'adjust_cart_item_price', 99, 2);
        $this->loader->add_filter('woocommerce_product_variation_get_price', $plugin_cart, 'adjust_cart_item_price', 99, 2);
        $this->loader->add_action('woocommerce_before_calculate_totals', $plugin_cart, 'before_calculate_totals', 99);
        $this->loader->add_filter('woocommerce_get_cart_item_from_session', $plugin_cart, 'adjust_cart_item_data', 99, 2);
        
        // Checkout
        $this->loader->add_filter('woocommerce_checkout_fields', $plugin_checkout, 'customize_checkout_fields');
        $this->loader->add_action('woocommerce_checkout_update_order_meta', $plugin_checkout, 'save_custom_checkout_fields');
        // Change the shipping fields heading
        $this->loader->add_filter('woocommerce_shipping_fields_heading', $plugin_checkout, 'change_shipping_fields_heading');
        // Change "Ship to a different address?" text
        $this->loader->add_filter('woocommerce_ship_to_different_address_text', $plugin_checkout, 'change_ship_to_different_address_label');
        // Add these lines for the enhanced checkout experience
        $this->loader->add_action('wp_head', $plugin_checkout, 'add_checkout_styles');
        $this->loader->add_action('wp_footer', $plugin_checkout, 'add_checkout_scripts');
        $this->loader->add_action('woocommerce_checkout_before_customer_details', $plugin_checkout, 'display_courier_info_on_checkout');
        // Add bank details field
        $this->loader->add_action('woocommerce_after_checkout_billing_form', $plugin_checkout, 'add_bank_details_fields');
        
        // Order
        $this->loader->add_action('woocommerce_checkout_order_processed', $plugin_order, 'process_courier_order', 10, 3);
        $this->loader->add_action('woocommerce_admin_order_data_after_shipping_address', $plugin_order, 'display_dimension_info_in_order', 10, 1);
        $this->loader->add_action('woocommerce_order_details_after_order_table', $plugin_order, 'display_dimension_info_in_order_details', 10, 1);
        // Display bank details on order pages
        $this->loader->add_action('woocommerce_order_details_after_order_table', $plugin_order, 'display_bank_details_in_order', 20);
        $this->loader->add_action('woocommerce_admin_order_data_after_billing_address', $plugin_order, 'display_bank_details_in_order', 20);
        
        // Emails
        $this->loader->add_action('woocommerce_email_after_order_table', $plugin_email, 'add_dimension_info_to_emails', 10, 4);
        // Add bank details to order emails
        $this->loader->add_action('woocommerce_email_after_order_table', $plugin_email, 'add_bank_details_to_emails', 20, 4);
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    CargoHub_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }
}