<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @since      1.0.0
 * @package    CargoHub
 */

if (!defined('WPINC')) {
    die;
}

/**
 * The public-facing functionality of the plugin.
 *
 * Handles all general public-facing functionality that isn't already
 * covered by the product, cart, checkout, or order classes.
 */
class CargoHub_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param    string    $plugin_name       The name of this plugin.
     * @param    string    $version           The version of this plugin.
     */
    public function __construct($plugin_name, $version) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {
        wp_enqueue_style(
            $this->plugin_name,
            CARGOHUB_PLUGIN_URL . 'assets/css/cargohub-public.css',
            array(),
            $this->version,
            'all'
        );
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {
        wp_enqueue_script(
            $this->plugin_name,
            CARGOHUB_PLUGIN_URL . 'assets/js/cargohub-public.js',
            array('jquery'),
            $this->version,
            true
        );
        
        // Localize the script with settings and translations
        $settings = get_option('cargohub_general_settings', array());
        
        $localized_data = array(
            'ajaxUrl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('cargohub-nonce'),
            'enableVolumetric' => isset($settings['enable_volumetric_weight']) ? $settings['enable_volumetric_weight'] === 'yes' : true,
            'volumetricDivisor' => isset($settings['volumetric_divisor']) ? (int) $settings['volumetric_divisor'] : 5000,
            'weightUnit' => isset($settings['weight_unit']) ? $settings['weight_unit'] : 'kg',
            'dimensionUnit' => isset($settings['dimension_unit']) ? $settings['dimension_unit'] : 'cm',
            'codFeePercentage' => isset($settings['cod_fee_percentage']) ? (float) $settings['cod_fee_percentage'] : 10,
            'currencySymbol' => get_woocommerce_currency_symbol(),
            'currencyPosition' => get_option('woocommerce_currency_pos'),
            'decimalSeparator' => wc_get_price_decimal_separator(),
            'thousandSeparator' => wc_get_price_thousand_separator(),
            'decimals' => wc_get_price_decimals(),
            'i18n' => array(
                'calculating' => __('Calculating price...', 'cargohub'),
                'errorCalculating' => __('Error calculating price. Please try again.', 'cargohub'),
                'priceCalculation' => __('Price Calculation', 'cargohub'),
                'shippingCost' => __('Shipping Cost', 'cargohub'),
                'codFee' => __('COD Fee', 'cargohub'),
                'totalCost' => __('Total Cost', 'cargohub'),
                'actualWeight' => __('Actual Weight', 'cargohub'),
                'volumetricWeight' => __('Volumetric Weight', 'cargohub'),
            )
        );
        
        wp_localize_script($this->plugin_name, 'cargoHubPublic', $localized_data);
    }

    /**
     * Calculate price via AJAX.
     *
     * @since    1.0.0
     */
    public function calculate_price() {
        // Check nonce
        check_ajax_referer('cargohub-nonce', 'security');
        
        // Get input values
        $weight = isset($_POST['weight']) ? (float) $_POST['weight'] : 0;
        $chargeable_weight = isset($_POST['chargeable_weight']) ? (float) $_POST['chargeable_weight'] : $weight;
        $service_type = isset($_POST['service_type']) ? sanitize_text_field($_POST['service_type']) : '';
        $cod_option = isset($_POST['cod_option']) ? sanitize_text_field($_POST['cod_option']) : 'no';
        
        // Basic validation
        if ($weight <= 0 || empty($service_type)) {
            wp_send_json_error(array('message' => __('Invalid input data.', 'cargohub')));
            return;
        }
        
        // Calculate shipping cost
        $shipping_cost = $this->calculate_shipping_cost($chargeable_weight, $service_type);
        
        // Apply COD fee if selected
        $cod_fee = 0;
        if ($cod_option === 'yes') {
            $options = get_option('cargohub_general_settings', array());
            $cod_fee_percentage = isset($options['cod_fee_percentage']) ? (float) $options['cod_fee_percentage'] : 10;
            
            $cod_fee = ($shipping_cost * $cod_fee_percentage) / 100;
        }
        
        // Calculate total cost
        $total_cost = $shipping_cost + $cod_fee;
        
        // Prepare response data
        $response_data = array(
            'weight' => $weight,
            'volumetric_weight' => isset($_POST['volumetric_weight']) ? (float) $_POST['volumetric_weight'] : 0,
            'chargeable_weight' => $chargeable_weight,
            'service_type' => $service_type,
            'cod_option' => $cod_option,
            'shipping_cost' => $shipping_cost,
            'cod_fee' => $cod_fee,
            'total_cost' => $total_cost
        );
        
        wp_send_json_success($response_data);
    }

    /**
     * Calculate the shipping cost based on weight and service type.
     *
     * @since    1.0.0
     * @param    float    $weight         The weight of the package.
     * @param    string   $service_type   The service type code.
     * @return   float                    The shipping cost.
     */
    private function calculate_shipping_cost($weight, $service_type) {
        global $wpdb;
        
        // Get customer's country
        $customer_country = WC()->customer->get_shipping_country();
        if (empty($customer_country)) {
            $customer_country = WC()->customer->get_billing_country();
        }
        
        // If still empty, use store base country
        if (empty($customer_country)) {
            $customer_country = WC()->countries->get_base_country();
        }
        
        // Find the zone for the customer's country
        $zones_table = $wpdb->prefix . 'cargohub_zones';
        $zone_rows = $wpdb->get_results("SELECT id, countries FROM $zones_table");
        
        $zone_id = 0;
        foreach ($zone_rows as $zone) {
            $countries = unserialize($zone->countries);
            if (in_array($customer_country, $countries)) {
                $zone_id = $zone->id;
                break;
            }
        }
        
        // If no zone found, use the first zone as default
        if ($zone_id === 0 && !empty($zone_rows)) {
            $zone_id = $zone_rows[0]->id;
        }
        
        // Get rate for this zone, service type, and weight
        $rates_table = $wpdb->prefix . 'cargohub_rates';
        $rate = $wpdb->get_row($wpdb->prepare(
            "SELECT price FROM $rates_table 
            WHERE zone_id = %d 
            AND service_type = %s 
            AND weight_from <= %f 
            AND weight_to > %f 
            ORDER BY price ASC 
            LIMIT 1",
            $zone_id,
            $service_type,
            $weight,
            $weight
        ));
        
        if ($rate) {
            return (float) $rate->price;
        }
        
        // If no specific rate found, find the highest weight range
        $highest_rate = $wpdb->get_row($wpdb->prepare(
            "SELECT price FROM $rates_table 
            WHERE zone_id = %d 
            AND service_type = %s 
            ORDER BY weight_to DESC 
            LIMIT 1",
            $zone_id,
            $service_type
        ));
        
        if ($highest_rate) {
            return (float) $highest_rate->price;
        }
        
        // If still no rate found, return a default price
        return 0;
    }
}